SystemOrganization addCategory: #'ToolsTests-Browser'!
SystemOrganization addCategory: #'ToolsTests-Debugger'!
SystemOrganization addCategory: #'ToolsTests-FileList'!
SystemOrganization addCategory: #'ToolsTests-Inspector'!

Inspector variableSubclass: #InspectorTestInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!InspectorTestInspector methodsFor: 'fields - streaming' stamp: 'mt 4/8/2020 14:27'!
streamFieldsOn: aStream
	
	self error.! !

StandardToolSet subclass: #DebuggerTestsToolSet
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Debugger'!

!DebuggerTestsToolSet methodsFor: 'debugging' stamp: 'mt 10/6/2019 17:53'!
debugProcess: aProcess context: aContext label: aString contents: contents fullView: aBool

	(data at: #debuggedProcesses ifAbsentPut: [OrderedCollection new])
		add: aProcess.
		
	aProcess suspend.! !

!DebuggerTestsToolSet methodsFor: 'error handling' stamp: 'mt 10/6/2019 13:44'!
doesNotUnderstand: aMessage

	aMessage numArgs = 0
		ifTrue: [^ data at: aMessage selector ifAbsent: [{}]].
	
	(self class respondsTo: aMessage selector)
		ifTrue: [^ aMessage sendTo: self class].
	
	^ super doesNotUnderstand: aMessage! !

!DebuggerTestsToolSet methodsFor: 'debugging - handlers' stamp: 'mt 10/5/2019 10:28'!
handleError: anError

	(data at: #handledErrors ifAbsentPut: [OrderedCollection new])
		add: anError.
		
	Processor activeProcess suspend.! !

!DebuggerTestsToolSet methodsFor: 'debugging - handlers' stamp: 'mt 10/6/2019 17:52'!
handleUserInterruptRequest: aString

	(data at: #handledUserInterruptRequests ifAbsentPut: [OrderedCollection new])
		add: aString.! !

!DebuggerTestsToolSet methodsFor: 'debugging - handlers' stamp: 'mt 10/6/2019 14:01'!
handleWarning: aWarning

	(data at: #handledWarnings ifAbsentPut: [OrderedCollection new])
		add: aWarning.
		
	Processor activeProcess suspend.! !

!DebuggerTestsToolSet methodsFor: 'initialize-release' stamp: 'mt 10/5/2019 10:25'!
initialize

	super initialize.
	data := Dictionary new.! !

ClassTestCase subclass: #FileListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-FileList'!

!FileListTest methodsFor: 'private' stamp: 'ul 1/11/2010 07:08'!
checkIsServiceIsFromDummyTool: service
	
	^ (service instVarNamed: #provider) = DummyToolWorkingWithFileList and: [
		service label = 'menu label' and: [
		(service instVarNamed: #selector) = #loadAFileForTheDummyTool: ] ]! !

!FileListTest methodsFor: 'initialize' stamp: 'SD 11/10/2001 21:48'!
setUp

	DummyToolWorkingWithFileList initialize.! !

!FileListTest methodsFor: 'initialize' stamp: 'SD 11/10/2001 21:49'!
tearDown

	DummyToolWorkingWithFileList unregister.! !

!FileListTest methodsFor: 'tests' stamp: 'tpr 1/7/2019 15:43'!
testAllRegisteredServices
	"(self selector: #testAllRegisteredServices) debug"

	"This should not throw an exception."
	FileServices allRegisteredServices.! !

!FileListTest methodsFor: 'tests' stamp: 'tpr 1/7/2019 15:44'!
testMenuReturned
	"(self selector: #testToolRegistered) debug"

	self assert: (FileServices registeredFileReaderClasses includes: DummyToolWorkingWithFileList)! !

!FileListTest methodsFor: 'tests' stamp: 'stephaneducasse 10/9/2005 20:50'!
testService
	"a stupid test to check that the class returns a service"
	"(self selector: #testService) debug"
	
	| service |
	service := (DummyToolWorkingWithFileList fileReaderServicesForFile: 'abab.kkk' suffix: 'kkk') first.
	self assert: (self checkIsServiceIsFromDummyTool: service).
	service := (DummyToolWorkingWithFileList fileReaderServicesForFile: 'zkk.gz' suffix: 'gz').
	self assert: service isEmpty! !

!FileListTest methodsFor: 'tests' stamp: 'pre 5/19/2016 14:50'!
testServicesForFileEnding
	"(self selector: #testServicesForFileEnding) debug"

	| items |
	items := (FileList new directory: FileDirectory default; yourself) 
	  itemsForFile: (FileDirectory default fullNameFor: 'aaa.kkk').
	self assert: (items anySatisfy: [ :ea | self checkIsServiceIsFromDummyTool: ea ]).
! !

!FileListTest methodsFor: 'tests' stamp: 'tpr 1/7/2019 15:44'!
testToolRegistered
	"(self selector: #testToolRegistered) debug"

	self assert: (FileServices registeredFileReaderClasses includes: DummyToolWorkingWithFileList)! !

!FileListTest methodsFor: 'tests' stamp: 'tpr 1/7/2019 15:44'!
testToolRegisteredUsingInterface
	"(self selector: #testToolRegisteredUsingInterface) debug"

	self assert: (FileServices isReaderNamedRegistered: #DummyToolWorkingWithFileList)! !

ClassTestCase subclass: #InspectorTest
	instanceVariableNames: 'inspector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

InspectorTest subclass: #BasicInspectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!BasicInspectorTest methodsFor: 'failures' stamp: 'mt 4/20/2020 09:47'!
expectedFieldExpressionFailures

	^ #('self')! !

!BasicInspectorTest methodsFor: 'support - error' stamp: 'mt 4/14/2020 14:49'!
selectInvalidField
	"Create a custom field. The existing fields will all work because the basic inspector shows only minimal information about the object."
	
	self
		during: [self inspector addCustomField]
		type: 'self printString'.! !

!BasicInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 14:21'!
testFieldList

	#(self 'class' 'size') do: [:label |
		self assertFieldVisible: label].! !

!BasicInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 14:41'!
testFieldSelf
	"The basic inspector sends as little messages as possible to the object-under-inspector. So, just look for the correct class name in a field's contents."

	| namePattern |
	namePattern := '*{1}*' format: { (thisContext objectClass: self object) name }.
	self assert: (self inspector fields anySatisfy: [:field | namePattern match: field value]).! !

InspectorTest subclass: #ClassInspectorTest
	instanceVariableNames: ''
	classVariableNames: 'InnerTestObject'
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!ClassInspectorTest methodsFor: 'running' stamp: 'mt 4/14/2020 14:57'!
createObject

	InnerTestObject := super createObject.
	^ self class! !

!ClassInspectorTest methodsFor: 'support - error' stamp: 'mt 4/14/2020 15:04'!
makeObjectInvalid

	InnerTestObject beInvalid.! !

!ClassInspectorTest methodsFor: 'support - error' stamp: 'mt 4/14/2020 15:05'!
selectInvalidField

	self inspector selectFieldSuchThat: [:field |
		field type = #classVar and: [field value == InnerTestObject]].! !

!ClassInspectorTest methodsFor: 'running' stamp: 'mt 4/14/2020 15:03'!
tearDown

	InnerTestObject := nil.
	super tearDown.! !

!ClassInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 15:07'!
testCustomField

	self
		during: [self inspector addCustomField]
		type: 'self packageInfo'.
	
	self assertFieldVisible: 'self packageInfo'.
	self assertFieldSelected: 'self packageInfo'.
	self assertValuePaneShows: '*ToolsTests*'.! !

!ClassInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 14:58'!
testFieldList
	
	#(self 'all inst vars' superclass InnerTestObject) do: [:label |
		self assertFieldVisible: label].! !

!ClassInspectorTest methodsFor: 'tests' stamp: 'mt 4/15/2020 09:59'!
testPoolDictionaries
	"All class inspectors should list the referenced pool dictionaries. Use an existing class from the base system that is known to rely of such a pool. If we would add our own references, loading these tests could raise extra dialog prompts."

	self denyFieldVisible: 'TextConstants'.
	self inspector object: TextStyle.
	self assertFieldVisible: 'TextConstants'.! !

!ClassInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 15:31'!
testValuePaneModify

	self inspector selectFieldNamed: 'InnerTestObject'.
	self assertFieldSelected: 'InnerTestObject'.
	
	self deny: 42 equals: InnerTestObject.
	self inValuePaneTypeAndAccept: '42'.
	self assert: 42 equals: InnerTestObject.! !

InspectorTest subclass: #CollectionInspectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!CollectionInspectorTest methodsFor: 'running' stamp: 'mt 4/8/2020 14:45'!
createObject

	^ OrderedCollection new
		addAll: #(discovery navel smith);
		yourself! !

!CollectionInspectorTest methodsFor: 'running' stamp: 'mt 4/7/2020 18:32'!
createObjectWithTruncation

	^ (1 to: 500) asOrderedCollection! !

!CollectionInspectorTest methodsFor: 'support' stamp: 'mt 4/15/2020 09:11'!
fieldTypeOfIndexedVariable

	^ #element! !

!CollectionInspectorTest methodsFor: 'support - error' stamp: 'mt 4/14/2020 16:45'!
makeObjectInvalid

	self object at: 1 put: InspectorTestObject newInvalid.
	self simulateStepping.! !

!CollectionInspectorTest methodsFor: 'support - error' stamp: 'mt 4/14/2020 15:35'!
selectInvalidField
	
	self inspector selectFieldNamed: '1'.! !

!CollectionInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 15:36'!
testAddElement
	"Add a new element through the Smalltalk expression '6 * 7' and check whether the field representing that new element will be selected automatically."

	| objectSize |
	objectSize := self object size.
	
	self assertFieldVisible: objectSize asString.
	self deny: (self object includes: 42).
	
	self
		during: [self inspector addElement]
		type: '6 * 7'. "42"

	self assert: (self object includes: 42).
		
	self assertFieldVisible: (objectSize + 1) asString.
	self assertFieldSelected: (objectSize + 1) asString.
	self assertValuePaneShows: '42'.! !

!CollectionInspectorTest methodsFor: 'tests - special' stamp: 'mt 4/14/2020 16:13'!
testAddElementError
	"Not all collections support addition or removal of elements."

	self class == CollectionInspectorTest ifFalse: [^ self "Pass the test automatically"].

	self inspector object: Array new.
	self
		should: [self inspector addElement: 1]
		raise: Error.! !

!CollectionInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 16:15'!
testAddElementMenu

	| testMenuEntry |
	testMenuEntry := [self fieldListMenu items anySatisfy: [:item | '*add*element*' match: item contents ]].

	self inspector selectField: nil.
	self assert: testMenuEntry.
	self inspector ensureSelectedField.
	self assert: testMenuEntry.! !

!CollectionInspectorTest methodsFor: 'tests - special' stamp: 'mt 4/15/2020 10:00'!
testAddElementMenuHidden

	| testMenuEntry |
	self class == CollectionInspectorTest ifFalse: [^ self "Pass the test automatically"].

	testMenuEntry := [self fieldListMenu items anySatisfy: [:item | '*add*element*' match: item contents ]].

	self inspector object: Array new.
	self deny: testMenuEntry.! !

!CollectionInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 15:59'!
testCustomField

	self
		during: [self inspector addCustomField]
		type: 'self take: 5'.
	
	self assertFieldVisible: 'self take: 5'.
	self assertFieldSelected: 'self take: 5'.
	self assertValuePaneShows: '*navel*'.! !

!CollectionInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 15:38'!
testFieldList

	#(self 'all inst vars') do: [:label | self assertFieldVisible: label].
	1 to: self object size do: [:index | self assertFieldVisible: index printString].! !

!CollectionInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 15:52'!
testRemoveElement
	"Remove an element from the collection-under-inspection by selecting any element's field first and then removing that selected element."
	
	| element |
	self assert: self object size > 1.
	self inspector selectFieldSuchThat: [:field | field type = #element].

	element := self inspector selection.
	self assert: (self object includes: element).
	
	self inspector removeSelection.
	self deny: (self object includes: element).

	"The next remaining element will automatically be selected."
	self assert: #element equals: self inspector selectedField type.
	self assert: (self object includes: self inspector selection).! !

!CollectionInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 17:47'!
testUninitialized
	"Single stepping through a debugger can observe the object state after creation but before initialization. Thus 'object size' may throw an exception for trying to do arithmetic on nil."
	
	self inspector selectFieldNamed: 'self'.
	self assertFieldSelected: 'self'.
	self assertValuePaneWorks.
	
	self inspector object: self object class basicNew.
	self assertFieldSelected: 'self'.
	self denyValuePaneWorks. "It's okay because the inspector is still working."! !

!CollectionInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 17:52'!
testValuePaneModify

	| overwrittenElement |
	self inspector selectFieldSuchThat: [:field | field type = #element].
	overwrittenElement := self inspector selection.

	self assert: (self object includes: overwrittenElement).
	self deny: (self object includes: #ontario).
		
	self inValuePaneTypeAndAccept: '#ontario'.
	self assertValuePaneShows: '#ontario'.
	
	self deny: (self object includes: overwrittenElement).
	self assert: (self object includes: #ontario).! !

CollectionInspectorTest subclass: #DictionaryInspectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!DictionaryInspectorTest methodsFor: 'running' stamp: 'mt 4/15/2020 09:16'!
createObject

	^ Dictionary withAll: {
		1 -> #discovery.
		7 -> #navel.
		11 -> #smith }! !

!DictionaryInspectorTest methodsFor: 'running' stamp: 'mt 4/15/2020 09:18'!
createObjectWithTruncation

	^ (1 to: 150)
		collect: [:number | number -> #genericApple]
		as: Dictionary! !

!DictionaryInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 17:29'!
testAddElement
	"The user will add a new key to the dictionary, not a value behind that key. That value needs to be set separatedly."

	self deny: (self object includesKey: 9).
	
	self
		during: [self inspector addElement]
		type: '3 ** 2'. "9"
	
	self assert: (self object includesKey: 9).
	
	self assertFieldVisible: '9'.
	self assertFieldSelected: '9'.
	
	self assertValuePaneShows: 'nil'.
	self inValuePaneTypeAndAccept: '#ontario'.
	self assertValuePaneShows: '#ontario'.
	
	self assert: #ontario equals: (self object at: 9).! !

!DictionaryInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 16:21'!
testFieldList
	"Check whether the most important fields are visible."
	
	#(self 'all inst vars' tally array) do: [:label |
		self assertFieldVisible: label].
	
	self object keysDo: [:key | self assertFieldVisible: key printString].! !

!DictionaryInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 17:29'!
testObjectChanged

	"1) Add a new key, which adds a new field to the list of fields."
	self denyFieldVisible: '9'.
	self object at: 9 put: nil.
	self denyFieldVisible: '9'.
	self simulateStepping.
	self assertFieldVisible: '9'.

	"2) Change the value behind the key, which changes the value-pane's contents."
	self inspector selectFieldNamed: '9'.
	self assertFieldSelected: '9'.
	self assertValuePaneShows: 'nil'.
	self object at: 9 put: #ontario.
	self assertValuePaneShows: 'nil'.
	self simulateStepping.
	self assertValuePaneShows: '#ontario'.! !

CollectionInspectorTest subclass: #SetInspectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!SetInspectorTest methodsFor: 'running' stamp: 'mt 4/14/2020 17:37'!
createObject

	^ Set new
		addAll: #(navel discovery smith boskoop);
		yourself
! !

!SetInspectorTest methodsFor: 'running' stamp: 'mt 4/15/2020 09:19'!
createObjectWithTruncation

	^ (1 to: 150) asSet! !

!SetInspectorTest methodsFor: 'support - error' stamp: 'mt 4/14/2020 17:45'!
makeObjectInvalid

	self object add: InspectorTestObject newInvalid.
	self simulateStepping.! !

!SetInspectorTest methodsFor: 'support - error' stamp: 'mt 4/14/2020 17:46'!
selectInvalidField

	self inspector selectFieldSuchThat: [:field |
		field type = #element and: [field value class == InspectorTestObject]].! !

!SetInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 17:35'!
testAddElement
	"Like super implementation but not checking the field names since we cannot now the exact indices of a set's internal array."

	self deny: (self object includes: 42).
	
	self
		during: [self inspector addElement]
		type: '6 * 7'. "42"

	self assert: (self object includes: 42).
	self assertValuePaneShows: '42'.! !

!SetInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 17:45'!
testFieldList

	#(self 'all inst vars') do: [:label | self assertFieldVisible: label].

	self object do: [:element | self assert: (
		self inspector fields anySatisfy: [:field |
			field type =#element and: [field value == element]] )]! !

!SetInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 17:40'!
testNil
	"Check proper use of a set's enclosed elements."

	self deny: (self object includes: nil).
	self deny: (self inspector fields anySatisfy: [:field | field value isNil]).
	
	self object add: nil.
	self simulateStepping.
	
	self assert: (self inspector fields anySatisfy: [:field | field value isNil]).! !

SetInspectorTest subclass: #WeakSetInspectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!WeakSetInspectorTest methodsFor: 'running' stamp: 'mt 4/14/2020 18:09'!
createObject

	| weakSet |
	weakSet := WeakSet withAll: super createObject.
	Smalltalk garbageCollect.
	^ weakSet! !

!WeakSetInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 18:07'!
testDebugConstruction
	"Ignore."! !

!WeakSetInspectorTest methodsFor: 'tests' stamp: 'mt 4/15/2020 09:57'!
testSymbolTableM6812
	"This test is related to http://bugs.squeak.org/view.php?id=6812. Check whether field selection and garbage collection are somehow interfering."
	
	| getRandomSymbol symbols priorContents currentContents currentIndex |
	self object removeAll.
	
	getRandomSymbol := [ 
		| token |
		token := (1 to: 10) collect: [:ea | ($a to: $z) atRandom] as: String.
		(Symbol lookup: token) ifNil: [token asSymbol] ifNotNil: [nil]].

	symbols := OrderedCollection new.
	10 timesRepeat: [
		getRandomSymbol value ifNotNil: [:symbol | symbols add: symbol]].	

	self object addAll: symbols.
	Smalltalk garbageCollect.
	self assert: symbols size equals: self object size.
	self assert: symbols notEmpty.
	
	1 to: symbols size do: [:round |
		currentIndex := 1.
		symbols removeLast.

		[(currentIndex := currentIndex + 1) <= self inspector fieldList size]
			whileTrue: [
				self inspector selectionIndex: currentIndex.
				self assert: priorContents ~= (currentContents := self inspector contents).
				priorContents := currentContents.

				Smalltalk garbageCollect. "Removes symbol from weak set"
				self simulateStepping. "Removes field from weak-set inspector"]].
	
	self assert: symbols isEmpty.
	self assert: self object isEmpty.! !

InspectorTest subclass: #CompiledCodeInspectorTest
	instanceVariableNames: ''
	classVariableNames: 'InnerTestObject'
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!CompiledCodeInspectorTest methodsFor: 'running' stamp: 'mt 5/15/2020 10:14'!
createObject
	"Note that we cannot return the block directly but require the indirection of #evaluate: because the resulting block will be modified during the tests. A block directly embedded in this #createObject method, however, would be re-used across test runs."
	
	InnerTestObject := super createObject.
	^ Compiler new
		evaluate: '[String withAll: #[67 97 114 112 101] "Carpe", #Squeak, InnerTestObject printString] method'
		in: nil
		to: self "Required for access to InnerTestObject"! !

!CompiledCodeInspectorTest methodsFor: 'running' stamp: 'mt 4/14/2020 11:32'!
evaluateObject

	^ (FullBlockClosure
		receiver: nil
		outerContext: nil
		method: self object
		copiedValues: nil) value! !

!CompiledCodeInspectorTest methodsFor: 'failures' stamp: 'mt 4/14/2020 12:05'!
expectedFieldExpressionFailures

	^ #('source code')! !

!CompiledCodeInspectorTest methodsFor: 'support - error' stamp: 'mt 4/8/2020 14:36'!
makeObjectInvalid

	InnerTestObject beInvalid.! !

!CompiledCodeInspectorTest methodsFor: 'support - error' stamp: 'mt 4/15/2020 10:02'!
selectInvalidField

	self inspector selectFieldSuchThat: [:field |
		field type = #literal and: [field value "binding" value == InnerTestObject]].! !

!CompiledCodeInspectorTest methodsFor: 'running' stamp: 'mt 4/8/2020 14:35'!
tearDown

	InnerTestObject := nil.
	super tearDown.! !

!CompiledCodeInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 11:35'!
testCustomField

	self
		during: [self inspector addCustomField]
		type: 'self allLiterals'.
	
	self assertFieldVisible: 'self allLiterals'.
	self assertFieldSelected: 'self allLiterals'.
	self assertValuePaneShows: '*#[*]*Squeak*'.! !

!CompiledCodeInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 11:49'!
testFieldList
	
	#(self '*bytecodes*' 'header' 'literal*') do: [:label |
		self assertFieldVisible: label].! !

!CompiledCodeInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 14:28'!
testValuePaneModify
	"Overridden to specify the kind of value to modify in a compiled-code object: the bytecodes. Looking at #createObject, we try to replace the pushConstant of the byte array at 34 with the one at 35. So, the beginning of the resulting string will change from 'CarpeSqueak' to 'SqueakSqueak'."
	
	self assert: 35 equals: (self object at: 38). "pushConstant: #[ ... ]"
	self assert: 36 equals: (self object at: 39). 	"pushConstant: #Squeak"
	self assert: (self evaluateObject beginsWith: 'CarpeSqueak').
		
	self inspector selectFieldNamed: '38'. "pushConstant: #[ ... ]"
	self assertFieldSelected: '38'.
	self inValuePaneTypeAndAccept: '36'. "pushConstant: #Squeak"

	self assert: 36 equals: (self object at: 38). "pushConstant: #Squeak"
	self assert: 36 equals: (self object at: 39). "pushConstant: #Squeak"	
	self assert: (self evaluateObject beginsWith: 'SqueakSqueak').! !

!CompiledCodeInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 14:28'!
testValuePaneModifyLiteral

	self inspector selectFieldSuchThat: [:field |
		field type = #literal and: [field value = #Squeak]].

	self assert: (self evaluateObject beginsWith: 'CarpeSqueak').
	self inValuePaneTypeAndAccept: '#Smalltalk'.	
	self assert: (self evaluateObject beginsWith: 'CarpeSmalltalk').! !

InspectorTest subclass: #ContextInspectorTest
	instanceVariableNames: ''
	classVariableNames: 'InnerTestObject'
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!ContextInspectorTest methodsFor: 'running' stamp: 'mt 4/14/2020 12:26'!
createObject

	InnerTestObject := super createObject.
	[[:arg1 :arg2 |
		| temp1 temp2 |
		temp1 := arg1 printString size - 1.
		temp2 := arg2 - 1.
		temp1 / temp2]
	value: InnerTestObject value: 1]
		on: Error do: [:error |
			^ error signalerContext sender copy
				push: 42;
				yourself].
	self error: 'Failed to set up context to inspect'! !

!ContextInspectorTest methodsFor: 'support - error' stamp: 'mt 4/8/2020 15:03'!
makeObjectInvalid

	InnerTestObject beInvalid.! !

!ContextInspectorTest methodsFor: 'support - error' stamp: 'mt 4/14/2020 12:26'!
selectInvalidField

	self inspector selectFieldSuchThat: [:field |
		field type = #tempVar and: [field value == InnerTestObject]].! !

!ContextInspectorTest methodsFor: 'running' stamp: 'mt 4/8/2020 15:01'!
tearDown

	InnerTestObject := nil.
	super tearDown.! !

!ContextInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 12:50'!
testCustomField

	self
		during: [self inspector addCustomField]
		type: 'self isDead'.
	
	self assertFieldVisible: 'self isDead'.
	self assertFieldSelected: 'self isDead'.
	self assertValuePaneShows: 'false'.! !

!ContextInspectorTest methodsFor: 'tests' stamp: 'ct 1/6/2020 00:48'!
testDebugConstruction

	self shouldntRaiseWhileDebugging: [
		[| foo | (foo := self) yourself] asContext]! !

!ContextInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 12:20'!
testFieldList
	"No need to look for the fields for temporaries because those will be covered in other tests, which is more useful. Only list the mandatory fields here."

	#(self 'all inst vars' 'sender' 'pc' 'stackp' 'method' 'closureOrNil' 'receiver')
		do: [:label | self assertFieldVisible: label].! !

!ContextInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 14:28'!
testValuePaneModify
	"Try to change the values of all arguments and temporary variables. Check if the object-under-inspection receives those changes."

	| testObjectFound |
	testObjectFound := false.
	
	self object tempNames doWithIndex: [:temp :index |
		| prior current input |
		self inspector selectFieldSuchThat: [:field | field type = #tempVar and: [field key = temp]].
		self assertFieldSelected: '*', temp, '*'. "allow bells and whistles"
		
		prior := self object namedTempAt: index.
		self assert: (prior isNumber or: [prior == InnerTestObject]).

		testObjectFound := testObjectFound or: [prior == InnerTestObject].
		current := (prior isNumber ifTrue: [prior + 1] ifFalse: [#smith]).
		input := prior isNumber ifTrue: [self inspector contents, ' +1'] ifFalse: ['#smith'].
		
		self deny: current equals: (self object namedTempAt: index).
		self inValuePaneTypeAndAccept: input.
		self assert: current equals: (self object namedTempAt: index)].
	
	self assert: testObjectFound.! !

ContextInspectorTest subclass: #ContextVariablesInspectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Debugger'!

!ContextVariablesInspectorTest methodsFor: 'failures' stamp: 'mt 4/14/2020 18:26'!
expectedFieldExpressionFailures

	^ #('stack top')! !

!ContextVariablesInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 18:23'!
testCustomField

	self
		during: [self inspector addCustomField]
		type: 'thisContext isDead'.
	
	self assertFieldVisible: 'thisContext isDead'.
	self assertFieldSelected: 'thisContext isDead'.
	self assertValuePaneShows: 'false'.! !

!ContextVariablesInspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 18:14'!
testFieldList

	#(thisContext 'stack top' 'all temp vars' '*arg*' '*temp*')
		do: [:label | self assertFieldVisible: label].! !

!ContextVariablesInspectorTest methodsFor: 'tests' stamp: 'mt 4/15/2020 10:05'!
testInspectorClass
	"This is inspector is a variation of regular context inspectors and is used in debuggers. So, after calling #inspect: the inspector class will indeed change to the regular one."
	
	| previousInspectorClass |
	self assert: self object inspectorClass ~~ self inspector class.
	previousInspectorClass := self inspector class.
	self inspector inspect: self object.
	self deny: previousInspectorClass equals: self inspector class.
	self assert: self object inspectorClass equals: self inspector class.! !

!InspectorTest class methodsFor: 'testing' stamp: 'ct 12/13/2019 17:24'!
shouldInheritSelectors

	^ true! !

!InspectorTest methodsFor: 'assertions - matching' stamp: 'ct 1/3/2020 23:55'!
assert: pattern matches: actual

	self
		assert: [(actual isString or: [actual isText]) and: [pattern match: actual]]
		description: [self comparingStringBetweenMatch: pattern and: actual]! !

!InspectorTest methodsFor: 'assertions - ui' stamp: 'mt 4/14/2020 12:44'!
assertFieldSelected: fieldName
	"Looking at the inspector's #fieldList, which contains the list of labels visible to the user, check whether that list contains fieldName, which is the label the user is looking for."

	self
		assert: (self inspector selectionIndex
			between: 1
			and: self inspector fieldList size);
		assert: fieldName 
		matches: (self inspector fieldList at: self inspector selectionIndex) asString.! !

!InspectorTest methodsFor: 'assertions - ui' stamp: 'mt 4/14/2020 11:48'!
assertFieldVisible: fieldNameOrPattern

	self assert: (self inspector fieldList anySatisfy: [:label |
					label asString = fieldNameOrPattern
						or: [fieldNameOrPattern match: label] ]).! !

!InspectorTest methodsFor: 'assertions' stamp: 'mt 4/8/2020 11:03'!
assertMenuWorks

	| aMenu |
	aMenu := self fieldListMenu.
	self
		assert: aMenu items notEmpty;
		assert: (aMenu items allSatisfy: [:item | item contents notEmpty]).! !

!InspectorTest methodsFor: 'assertions - ui' stamp: 'mt 4/8/2020 10:35'!
assertNoFieldSelected

	self assert: self inspector selectionIndex = 0.! !

!InspectorTest methodsFor: 'assertions - ui' stamp: 'mt 4/7/2020 17:54'!
assertValuePaneShows: contents

	self assert: contents matches: self inspector contents.! !

!InspectorTest methodsFor: 'assertions' stamp: 'mt 4/14/2020 15:10'!
assertValuePaneWorks

	^ self denyValuePaneShows: '<*error*debug*>'! !

!InspectorTest methodsFor: 'assertions - matching' stamp: 'ct 1/12/2020 17:08'!
comparingStringBetweenMatch: pattern and: actual

	^ 'Pattern {1} is not matched by actual {2}' format: (
		{pattern. actual} collect: [:arg | arg printStringLimitedTo: 10])! !

!InspectorTest methodsFor: 'running' stamp: 'mt 4/7/2020 17:51'!
createObject

	^ InspectorTestObject new
		apple: #discovery;
		orange: #navel;
		yourself! !

!InspectorTest methodsFor: 'running' stamp: 'mt 4/15/2020 09:09'!
createObjectWithTruncation
	"Create an object that will surely trigger truncation of inspector fields when being inspected."

	^ (InspectorTestObject new: 500)
		apple: #discovery;
		orange: #navel;
		yourself! !

!InspectorTest methodsFor: 'assertions - matching' stamp: 'ct 2/14/2020 17:49'!
deny: pattern matches: actual

	self
		deny: [(actual isString or: [actual isText]) and: [pattern match: actual]]
		description: ['Actually matches {1}' format: {pattern}]! !

!InspectorTest methodsFor: 'assertions - ui' stamp: 'mt 4/7/2020 17:54'!
denyFieldSelected: fieldName

	self deny: (self inspector fieldList at: self inspector selectionIndex) asString = fieldName.! !

!InspectorTest methodsFor: 'assertions - ui' stamp: 'mt 4/7/2020 17:54'!
denyFieldVisible: fieldName

	self assert: (self inspector fieldList noneSatisfy: [:label |
					label asString = fieldName]).! !

!InspectorTest methodsFor: 'assertions - ui' stamp: 'mt 4/8/2020 10:35'!
denyNoFieldSelected

	self assert: self inspector selectionIndex > 0.! !

!InspectorTest methodsFor: 'assertions - ui' stamp: 'mt 4/7/2020 17:54'!
denyValuePaneShows: contents

	self deny: contents matches: self inspector contents.! !

!InspectorTest methodsFor: 'assertions' stamp: 'mt 4/14/2020 11:26'!
denyValuePaneWorks

	^ self assertValuePaneShows: '*error*debug*'! !

!InspectorTest methodsFor: 'support - ui' stamp: 'mt 4/14/2020 15:43'!
during: block confirm: boolean
	"When evaluating the block, there will be a dialog showing up that requests the user to confirm something. Simulate that interaction here."

	block valueSupplyingAnswer: {'*'. boolean}.! !

!InspectorTest methodsFor: 'support - ui' stamp: 'mt 4/8/2020 10:39'!
during: block type: expression
	"When evaluating the block, there will be a dialog showing up that requests the user to type something. Simulate that interaction here."

	block valueSupplyingAnswer: {'*'. expression}.! !

!InspectorTest methodsFor: 'failures' stamp: 'mt 4/14/2020 12:04'!
expectedFieldExpressionFailures
	"List all fields whose 'Get field expression' feature does not yet work."

	^ #()! !

!InspectorTest methodsFor: 'support' stamp: 'mt 4/7/2020 18:39'!
fieldListMenu

	^ self inspector fieldListMenu: MenuMorph new! !

!InspectorTest methodsFor: 'support' stamp: 'mt 4/15/2020 09:11'!
fieldTypeOfIndexedVariable

	^ #indexed! !

!InspectorTest methodsFor: 'support - ui' stamp: 'mt 4/8/2020 10:40'!
inValuePaneTypeAndAccept: aString
	"The user types aString in the value pane and accepts those contents."

	self inspector contents: aString notifying: nil.! !

!InspectorTest methodsFor: 'accessing' stamp: 'mt 4/7/2020 17:22'!
inspector

	^ inspector! !

!InspectorTest methodsFor: 'support - error' stamp: 'mt 4/8/2020 14:25'!
makeObjectInvalid
	"Violate some contract so that the inspector cannot call #printString on some field anymore without getting an error."

	self object beInvalid.! !

!InspectorTest methodsFor: 'accessing' stamp: 'mt 4/7/2020 17:21'!
object

	^ self inspector object! !

!InspectorTest methodsFor: 'support - error' stamp: 'mt 4/8/2020 14:41'!
selectInvalidField

	self inspector selectFieldNamed: 'self'.! !

!InspectorTest methodsFor: 'running' stamp: 'mt 4/7/2020 17:27'!
setUp

	super setUp.
	inspector := self targetClass on: self createObject.! !

!InspectorTest methodsFor: 'assertions' stamp: 'mt 4/7/2020 16:14'!
shouldntRaiseWhileDebugging: aBlock

	aBlock newProcess runUntil: [:ctxt |
		self
			shouldnt: [inspector inspect: ctxt receiver]
			raise: Error, Warning, Halt.
		false].! !

!InspectorTest methodsFor: 'support - ui' stamp: 'mt 4/14/2020 16:38'!
simulateStepping

	self inspector stepAt: 0 in: nil.! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/7/2020 17:53'!
testCustomField

	self
		during: [self inspector addCustomField]
		type: 'self fruits'.
	
	self assertFieldVisible: 'self fruits'.
	self assertFieldSelected: 'self fruits'.
	self assertValuePaneShows: '*discovery*navel*'.! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 15:43'!
testCustomFieldRemove

	self assert: 0 equals: self inspector customFields size.
	self during: [self inspector addCustomField] type: 'self'.
	self assert: 1 equals: self inspector customFields size.

	self assert: self inspector selectedField type = #custom.
	self during: [self inspector removeSelection] confirm: true.
	self assert: 0 equals: self inspector customFields size.! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/7/2020 17:31'!
testDebugConstruction

	self shouldntRaiseWhileDebugging: [
		self createObject]! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 12:03'!
testExpressions
	"All fields should provide an evaluable expression to be evaluated on the inspected objects to retrieve that field's value. Try to test that by re-setting that expression as a getter and compare the resulting values."

	self inspector fields
		reject: [:field | self expectedFieldExpressionFailures includes: field name asString]
		thenDo: [:field |
			| expression content |
			(expression := self inspector expressionForField: field) ifNil: [self fail].
			content := field getValueFor: self inspector. 		
			field setGetterFor: self inspector to: expression ifFail: [self fail].
			self assert: content equals: (field getValueFor: self inspector)].! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 11:39'!
testFieldList
	"Check whether the most important fields are visible."
	
	#(self 'all inst vars' apple orange) do: [:label |
		self assertFieldVisible: label].! !

!InspectorTest methodsFor: 'tests - special' stamp: 'mt 4/15/2020 09:10'!
testFieldListError
	"Choose an inspector that messes up field streaming. Check whether the field list shows still something informative."

	self class == InspectorTest ifFalse: [^ self "Pass the test automatically."].
	
	inspector := InspectorTestInspector on: Object new.
	
	self assertFieldVisible: '<error>'.
	inspector ensureSelectedField.
	self assertFieldSelected: '<error>'.
	self assertValuePaneShows: '*error*debug*'.! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 11:20'!
testFieldListMenu
	"Select one field after another and check whether the menu can be invoked."

	self assertNoFieldSelected.
	self assertMenuWorks.

	1 to: self inspector fieldList size do: [:index |
		self inspector selectionIndex: index.
		self denyNoFieldSelected.
		self assertMenuWorks].
	! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 14:24'!
testFieldSelf
	"There should be at least one field pointing to the inspected object itself."

	self assert: (self inspector fields anySatisfy: [:field | field value == self object]).! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/8/2020 10:28'!
testInspectorClass
	"Be sure to use the correct inspector for our object-under-inspection. If this test fails, #targetClass or #setUp might be wrong."

	| previousInspectorClass |
	previousInspectorClass := self inspector class.
	self inspector inspect: self object.
	self assert: previousInspectorClass equals: self inspector class.! !

!InspectorTest methodsFor: 'tests - special' stamp: 'mt 4/15/2020 09:09'!
testObjectChanged
	
	self class == InspectorTest ifFalse: [^ self "Pass the test automatically."].
	
	self inspector selectFieldNamed: 'self'.
	self denyValuePaneShows: '*ontario*'.
	
	self object apple: #ontario.
	self denyValuePaneShows: '*ontario*'.
	
	self simulateStepping.
	self assertValuePaneShows: '*ontario*'.! !

!InspectorTest methodsFor: 'tests - special' stamp: 'mt 4/14/2020 16:04'!
testTruncationEllipsis
	"Even the most generic inspector supports truncation of indexed variables."

	| ellipsis |
	(self class includesSelector: #createObjectWithTruncation)
		ifFalse: [^ self "Run this test only if explicitely refined."].
	
	self inspector object: self createObjectWithTruncation.
	self assert: self inspector class = self targetClass. "No change."
	
	self assert: self inspector fields size >= self inspector truncationLimit.
	self assertFieldVisible: '...'.

	self inspector selectFieldSuchThat: [:field | field type = #ellipsis].
	ellipsis := self inspector selectedField.

	self assert: '*...*' matches: ellipsis name.
	self assertValuePaneShows: '*not shown*'.
	
	self inspector fields do: [:field |
		"All visible elements are from that object."
		self assert: (field type = #element) ==> [self inspector object includes: field value]].! !

!InspectorTest methodsFor: 'tests - special' stamp: 'mt 4/14/2020 11:38'!
testTruncationEllipsisMenu

	(self class includesSelector: #createObjectWithTruncation)
		ifFalse: [^ self "Run this test only if explicitely refined."].
		
	self inspector object: self createObjectWithTruncation.
	self inspector selectFieldSuchThat: [:field | field type = #ellipsis].
	self assertMenuWorks.! !

!InspectorTest methodsFor: 'tests - special' stamp: 'mt 4/15/2020 09:12'!
testTruncationTail

	| ellipsisIndex firstElementIndex |
	(self class includesSelector: #createObjectWithTruncation)
		ifFalse: [^ self "Run this test only if explicitely refined."].
		
	self inspector object: self createObjectWithTruncation.
	
	firstElementIndex := self inspector fields
		findFirst: [:field | field type = self fieldTypeOfIndexedVariable].
	ellipsisIndex := self inspector fields
		findFirst: [:field | field type = #ellipsis].

	self
		assert: self inspector truncationLimit
		equals: ellipsisIndex - firstElementIndex + 1 + self inspector truncationTail;
		assert: self inspector truncationTail
		equals: self inspector fields size - ellipsisIndex.! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 11:37'!
testValuePane
	"Select one field after another and check whether the value pane shows non-error contents."

	self assertNoFieldSelected.
	self assertValuePaneWorks.

	1 to: self inspector fieldList size do: [:index |
		self inspector selectionIndex: index.
		self denyNoFieldSelected.
		self assertValuePaneWorks].! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 11:27'!
testValuePaneError

	self makeObjectInvalid.
	self assertValuePaneWorks.
	
	self selectInvalidField.
	self denyValuePaneWorks.! !

!InspectorTest methodsFor: 'tests' stamp: 'mt 4/14/2020 15:02'!
testValuePaneModify

	self inspector selectFieldNamed: #apple.

	self deny: #ontario equals: self object apple.
	self assertValuePaneShows: '#discovery'.
		
	self inValuePaneTypeAndAccept: '#ontario'.

	self assert: #ontario equals: self object apple.
	self assertValuePaneShows: '#ontario'.! !

TestCase subclass: #BrowseTest
	instanceVariableNames: 'originalBrowserClass originalHierarchyBrowserClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!BrowseTest methodsFor: 'private' stamp: 'mu 3/6/2004 15:41'!
currentBrowsers
	^ (ActiveWorld submorphs
		select: [:each | (each isKindOf: SystemWindow)
				and: [each model isKindOf: Browser]]) asSet! !

!BrowseTest methodsFor: 'private' stamp: 'mu 3/11/2004 15:52'!
currentHierarchyBrowsers
	^ (ActiveWorld submorphs
		select: [:each | (each isKindOf: SystemWindow)
				and: [each model isKindOf: HierarchyBrowser]]) asSet! !

!BrowseTest methodsFor: 'private' stamp: 'mu 3/6/2004 15:27'!
ensureMorphic
	self isMorphic ifFalse: [self error: 'This test should be run in Morphic'].! !

!BrowseTest methodsFor: 'private' stamp: 'mu 3/6/2004 15:26'!
isMorphic
	^Smalltalk isMorphic! !

!BrowseTest methodsFor: 'running' stamp: 'mu 3/11/2004 15:57'!
setUp
	| systemNavigation |
	systemNavigation := SystemNavigation default.
	originalBrowserClass := systemNavigation browserClass.
	originalHierarchyBrowserClass := systemNavigation hierarchyBrowserClass.
	
	 systemNavigation browserClass: nil.
	 systemNavigation hierarchyBrowserClass: nil.
	
	! !

!BrowseTest methodsFor: 'running' stamp: 'mu 3/11/2004 15:57'!
tearDown
	| systemNavigation |
	systemNavigation := SystemNavigation default.
	 systemNavigation browserClass: originalBrowserClass.
	 systemNavigation hierarchyBrowserClass: originalHierarchyBrowserClass.! !

!BrowseTest methodsFor: 'tests' stamp: 'eem 11/16/2016 18:06'!
testBrowseClass
	"self debug: #testBrowseClass"
	| browsersBefore browsersAfter opened |
	self ensureMorphic.
	
	browsersBefore := self currentBrowsers.
	self withImmediatelyOpeningWindowsDo: [1 class browse].
	browsersAfter := self currentBrowsers.
	
	self assert:  (browsersAfter size  = (browsersBefore size + 1)).
	opened := browsersAfter removeAll: browsersBefore; yourself.
	self assert:  (opened size = 1).
	opened := opened asArray first.
	self assert: (opened model selectedClass == SmallInteger).
	
	opened delete
	
	
	! !

!BrowseTest methodsFor: 'tests' stamp: 'eem 11/16/2016 18:06'!
testBrowseHierarchyClass
	"self debug: #testBrowseHierarchyClass"
	| browsersBefore browsersAfter opened |
	self ensureMorphic.
	
	browsersBefore := self currentHierarchyBrowsers.
	self withImmediatelyOpeningWindowsDo: [1 class browseHierarchy].
	browsersAfter := self currentHierarchyBrowsers.
	
	self assert:  (browsersAfter size  = (browsersBefore size + 1)).
	opened := browsersAfter removeAll: browsersBefore; yourself.
	self assert:  (opened size = 1).
	opened := opened asArray first.
	self assert: (opened model selectedClass == SmallInteger).
	
	opened delete
	
	
	! !

!BrowseTest methodsFor: 'tests' stamp: 'eem 11/16/2016 18:06'!
testBrowseHierarchyInstance
	"self debug: #testBrowseHierarchyInstance"
	| browsersBefore browsersAfter opened |
	self ensureMorphic.
	
	browsersBefore := self currentHierarchyBrowsers.
	self withImmediatelyOpeningWindowsDo: [1 browseHierarchy].
	browsersAfter := self currentHierarchyBrowsers.
	
	self assert:  (browsersAfter size  = (browsersBefore size + 1)).
	opened := browsersAfter removeAll: browsersBefore; yourself.
	self assert:  (opened size = 1).
	opened := opened asArray first.
	self assert: (opened model selectedClass == SmallInteger).
	
	opened delete
	
	
	! !

!BrowseTest methodsFor: 'tests' stamp: 'eem 6/8/2017 11:48'!
testBrowseHierarchyMetaclass
	"self debug: #testBrowseHierarchyMetaclass"
	| browsersBefore browsersAfter opened |
	self ensureMorphic.
	
	browsersBefore := self currentHierarchyBrowsers.
	self withImmediatelyOpeningWindowsDo: [1 class class browseHierarchy].
	browsersAfter := self currentHierarchyBrowsers.
	
	self assert:  (browsersAfter size  = (browsersBefore size + 1)).
	opened := browsersAfter removeAll: browsersBefore; yourself.
	self assert:  (opened size = 1).
	opened := opened asArray first.
	self assert: (opened model selectedClass == Metaclass).
	
	opened delete
	
	
	! !

!BrowseTest methodsFor: 'tests' stamp: 'eem 11/16/2016 18:07'!
testBrowseInstance
	"self debug: #testBrowseInstance"
	| browsersBefore browsersAfter opened |
	self ensureMorphic.
	
	browsersBefore := self currentBrowsers.
	self withImmediatelyOpeningWindowsDo: [1 browse].
	browsersAfter := self currentBrowsers.
	
	self assert:  (browsersAfter size  = (browsersBefore size + 1)).
	opened := browsersAfter removeAll: browsersBefore; yourself.
	self assert:  (opened size = 1).
	opened := opened asArray first.
	self assert: (opened model selectedClass == SmallInteger).
	
	opened delete
	
	
	! !

!BrowseTest methodsFor: 'tests' stamp: 'eem 11/16/2016 18:07'!
testBrowseMetaclass
	"self debug: #testBrowseMetaclass"
	| browsersBefore browsersAfter opened |
	self ensureMorphic.
	
	browsersBefore := self currentBrowsers.
	self withImmediatelyOpeningWindowsDo: [1 class class browse].
	browsersAfter := self currentBrowsers.
	
	self assert:  (browsersAfter size  = (browsersBefore size + 1)).
	opened := browsersAfter removeAll: browsersBefore; yourself.
	self assert:  (opened size = 1).
	opened := opened asArray first.
	self assert: (opened model selectedClass == SmallInteger).
	
	opened delete
	
	
	! !

!BrowseTest methodsFor: 'tests' stamp: 'eem 5/23/2011 17:02'!
testFindReferencesToFloatNaN
	self assert: (SystemNavigation new allCallsOn: (Float bindingOf: #NaN)) notEmpty! !

!BrowseTest methodsFor: 'tests' stamp: 'eem 7/25/2011 10:21'!
testSystemNavigationAllBehaviorsDo
	| oc |
	oc := OrderedCollection new.
	SystemNavigation new allBehaviorsDo:
		[:c| oc add: c].
	self assert: oc size = oc asSet size! !

!BrowseTest methodsFor: 'private' stamp: 'eem 11/16/2016 18:05'!
withImmediatelyOpeningWindowsDo: aBlock
	"Turn off any preferences that defer to the user to open a window."
	| openToolsAttachedToMouseCursor |
	(UIManager default respondsTo: #openToolsAttachedToMouseCursor:) ifFalse:
		[^aBlock value].

	openToolsAttachedToMouseCursor := UIManager default openToolsAttachedToMouseCursor.
	UIManager default openToolsAttachedToMouseCursor: false.
	^aBlock ensure:
		[UIManager default openToolsAttachedToMouseCursor: openToolsAttachedToMouseCursor]! !

TestCase subclass: #BrowserHierarchicalListTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!BrowserHierarchicalListTest commentStamp: 'rkrk 8/24/2009 05:11' prior: 0!
Tests the optional hierarchical class ordering of Browser.!

!BrowserHierarchicalListTest methodsFor: 'assertion' stamp: 'rkrk 8/24/2009 05:03'!
assertCorrectOrderOf: classB followedBy: classA in: classCollection

	"classB comes before classA. Assert that classB is a superclass of classB or that 
	a common superclass is in front of both"
	| commonSuperclasses commonSuperclass classAIndex classBIndex superIndex |
	classA == classB ifTrue: [^ self].
	(classA inheritsFrom: classB) ifTrue: [^ self].
	commonSuperclasses := classA withAllSuperclasses intersection: classB withAllSuperclasses.
	commonSuperclass := commonSuperclasses first.
	(classCollection includes: commonSuperclass) ifFalse: [^ self].
	classAIndex := classCollection indexOf: classA.
	classBIndex := classCollection indexOf: classB.
	superIndex := classCollection indexOf: commonSuperclass.
	(superIndex < classAIndex and: [superIndex < classBIndex]) ifTrue: [^self].
	self fail.! !

!BrowserHierarchicalListTest methodsFor: 'helper' stamp: 'fbs 3/5/2011 20:55'!
hierarchicalClassListForCategory: category

	^ Browser new
		selectSystemCategory: category;
		hierarchicalClassList.! !

!BrowserHierarchicalListTest methodsFor: 'helper' stamp: 'rkrk 8/24/2009 04:32'!
nameToClass: classNameWithIndent

	^ Smalltalk classNamed: classNameWithIndent withoutLeadingBlanks asSymbol! !

!BrowserHierarchicalListTest methodsFor: 'tests' stamp: 'rkrk 8/24/2009 05:10'!
testListClassesHierarchically1

	| result classes category |
	category := 'Collections-Abstract'.
	result := self hierarchicalClassListForCategory: category.
	self assert: (SystemOrganization listAtCategoryNamed: category) size equals: result size.
	classes := result collect: [:ea | self nameToClass: ea].
	classes withIndexDo: [:ea : i |
		classes 
			from: 1 to: i
			do: [:other | self assertCorrectOrderOf: other followedBy: ea in: classes]].! !

!BrowserHierarchicalListTest methodsFor: 'tests' stamp: 'rkrk 8/24/2009 05:09'!
testListClassesHierarchically2

	| result classes category |
	category := 'Tools-Browser'.
	result := self hierarchicalClassListForCategory: category.
	self assert: (SystemOrganization listAtCategoryNamed: category) size equals: result size.
	classes := result collect: [:ea | self nameToClass: ea].
	classes withIndexDo: [:ea : i |
		classes 
			from: 1 to: i
			do: [:other | self assertCorrectOrderOf: other followedBy: ea in: classes]].! !

!BrowserHierarchicalListTest methodsFor: 'tests' stamp: 'nice 12/26/2009 00:33'!
testListClassesHierarchicallyIndent

	| result dict  |
	result := self hierarchicalClassListForCategory: 'Tools-Browser'.
	"Create class->indent mapping"
	dict := result inject: Dictionary new into: [:classIndentMapping :className |
		| indent |
		indent := className count: [:char | char = Character space or: [char = Character tab]].
		classIndentMapping at: (self nameToClass: className) put: indent.
		classIndentMapping].
	"assert that indent of class is larger than indent of superclass"
	dict keysAndValuesDo: [:class :myIndent |
		dict at: class superclass ifPresent: [:superIndent |
			self assert: myIndent > superIndent]].! !

TestCase subclass: #BrowserTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!BrowserTest commentStamp: 'fbs 3/4/2011 07:22' prior: 0!
I provide the test suite for Browser.
!

!BrowserTest methodsFor: 'running' stamp: 'fbs 3/3/2011 12:41'!
setUp
	browser := Browser new.! !

!BrowserTest methodsFor: 'tests' stamp: 'eem 9/24/2018 22:10'!
testAlphabetizeMessageCategories
	| categories |
	browser selectSystemCategory: browser class category.
	self deny: browser alphabetizeMessageCategories.
	
	browser selectClass: browser class.
	browser metaClassIndicated: false.
	self assert: browser alphabetizeMessageCategories.
	categories := browser classOrMetaClassOrganizer categories.
	self assert: (categories select: [:c| c first == $*]) isSorted.
	self assert: (categories select: [:c| c beginsWith: 'private']) isSorted.
	self assert: (categories reject: [:c| c first == $* or: [c beginsWith: 'private']]) isSorted.
	
	browser metaClassIndicated: true.
	self assert: browser alphabetizeMessageCategories.
	self assert: browser classOrMetaClassOrganizer categories isSorted.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/14/2011 17:44'!
testAlphabetizeMessageCategoriesLeavesClassSelected
	browser selectSystemCategory: browser class category.
	browser selectClassNamed: browser class name.
	
	"This is a no-op."
	browser alphabetizeMessageCategories.
	self assert: browser selectedClassName = browser class name.! !

!BrowserTest methodsFor: 'tests - message list' stamp: 'fbs 5/1/2011 08:54'!
testAlphabetizingMethodsPreservesPlace
	| oldCategory oldMethod |
	oldCategory := 'breakpoints'.
	oldMethod := #toggleBreakOnEntry.
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser selectMessageCategoryNamed: oldCategory.
	browser selectMessageNamed: oldMethod.
	browser alphabetizeMessageCategories.

	self assert: browser selectedMessageCategoryName = oldCategory.
	self assert: browser selectedMessageName = oldMethod.! !

!BrowserTest methodsFor: 'tests - message list' stamp: 'fbs 5/1/2011 09:06'!
testAlphabetizingMethodsPreservesPlaceWhenNoMessageCategorySelected
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser selectMessageCategoryNamed: nil.
	browser selectMessageNamed: nil.
	browser alphabetizeMessageCategories.

	self assert: browser selectedMessageCategoryName = nil.
	self assert: browser selectedMessageName = nil.! !

!BrowserTest methodsFor: 'tests - message list' stamp: 'fbs 5/1/2011 09:03'!
testAlphabetizingMethodsPreservesPlaceWhenNoMethodSelected
	| oldCategory |
	oldCategory := 'breakpoints'.
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser selectMessageCategoryNamed: oldCategory.
	browser selectMessageNamed: nil.
	browser alphabetizeMessageCategories.

	self assert: browser selectedMessageCategoryName = oldCategory.
	self assert: browser selectedMessageName = nil.! !

!BrowserTest methodsFor: 'tests' stamp: 'eem 6/16/2018 13:46'!
testBuildMessageCategoryBrowserEditString
	| spawn |
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser messageCategoryListIndex: 1.
	browser messageListIndex: 1.
	
	spawn := browser buildMessageCategoryBrowserEditString: 'We don''t yet test that we do the right thing with this string!!'.
	[self assert: spawn selectedSystemCategory = browser selectedSystemCategory.
	self assert: spawn selectedClass = browser selectedClass.
	self assert: spawn messageCategoryListIndex = browser messageCategoryListIndex.
	self assert: spawn messageListIndex = browser messageListIndex]
		ensure: [ [ToolBuilder default close: spawn topView] valueSupplyingAnswer: true].! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:52'!
testBuildMessageCategoryBrowserEditStringNoCategorySelected
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser messageCategoryListIndex: 0.
	
	self assert: (browser buildMessageCategoryBrowserEditString: 'foo') isNil.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/14/2011 17:42'!
testChangeMessageCategoriesLeavesClassSelected
	browser selectSystemCategory: browser class category.
	browser selectClassNamed: browser class name.
	
	"This is a no-op."
	browser changeMessageCategories: browser classOrMetaClassOrganizer printString.
	self assert: browser selectedClassName = browser class name.! !

!BrowserTest methodsFor: 'tests' stamp: 'ul 8/14/2015 18:00'!
testClassCommentAnnotation
	| annotation |
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	annotation := browser annotationForClassCommentFor: browser class.
	self assert: (annotation includesSubstring: browser class organization commentStamp).
	self assert: (annotation includesSubstring: 'class comment for').
	self assert: (annotation includesSubstring: browser className).! !

!BrowserTest methodsFor: 'tests' stamp: 'ul 8/14/2015 18:00'!
testClassCommentAnnotationIgnoresParameter
	| annotation |
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	"And the method ignores the parameter:"
	annotation := browser annotationForClassCommentFor: browser class superclass.
	self assert: (annotation includesSubstring: browser className).! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/31/2011 09:41'!
testClassCommentViewingPersists
	browser selectSystemCategory: browser class category.
	self assert: browser classList size > 1 description: 'We use the first TWO classes in the category!!'.
	browser classListIndex: 1.
	browser metaClassIndicated: false.
	browser editComment.
	
	browser classListIndex: 2.
	self assert: browser editSelection = #editComment.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 1/3/2013 22:48'!
testClassDefinitionAnnotation
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	self assert: browser class comment equals: (browser annotationForClassDefinitionFor: browser class).
	"And the method ignores the parameter:"
	self assert: browser class comment equals: (browser annotationForClassDefinitionFor: browser class superclass).! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 1/3/2013 20:26'!
testContentsNewClass
	browser selectSystemCategory: browser class category.

	self assert: (Class template: browser selectedSystemCategory) equals: browser contents.
	
	self flag: #todo. "I don't know how to test the other half of this: see Browser >> #newClassContents".! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/17/2011 07:55'!
testContentsSelection
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.

	"No category or message selected."
	self assert: browser contentsSelection isEmpty.

	"Category selected, but no message selected."
	browser messageCategoryListIndex: 1.
	browser selectMessageNamed: browser messageList first.
	self assert: browser contentsSelection isEmpty.
	
	"Category and message selected."
	browser selectMessageNamed: nil.
	self deny: browser contentsSelection isEmpty.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/12/2011 22:11'!
testCreateHierarchyTreeOf
	"Empty."
	self assert: (browser createHierarchyTreeOf: #()) isEmpty.
	"Single class."
	self assert: { Browser -> #() } = (browser createHierarchyTreeOf: { Browser }).
	"Single class + 1 subclass"
	self assert: { Browser->{HierarchyBrowser->#()} } = (browser createHierarchyTreeOf: { Browser. HierarchyBrowser. }).
	"Single class + 2 subclasses"
	self assert: { Browser->{HierarchyBrowser->#() . FileContentsBrowser->#()} } = (browser createHierarchyTreeOf: { Browser. HierarchyBrowser. FileContentsBrowser. }).
	"Superclass, class, subclass"
	self assert: { CodeHolder->{Browser->{HierarchyBrowser->#()}} } = (browser createHierarchyTreeOf: { CodeHolder. Browser. HierarchyBrowser. }).
	"Two 'unrelated' classes"
	self assert: {Browser->#() . SmallInteger->#()} = (browser createHierarchyTreeOf: { Browser. SmallInteger }).
	"Two 'unrelated' classes and a common ancestor"
	self assert: {Browser->#() . SmallInteger->#() . ProtoObject->#()} = (browser createHierarchyTreeOf: { Browser. SmallInteger. ProtoObject }).! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:25'!
testDefaultClassList
	browser selectSystemCategory: nil.
	self assert: browser defaultClassList isEmpty.
	
	browser selectSystemCategory: Browser category.
	self assert: browser defaultClassList = (SystemOrganization listAtCategoryNamed: Browser category)! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/30/2011 08:45'!
testFileOutMessageCategories
	| fileName |	
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser messageCategoryListIndex: 1.
	
	"Handcrafted from ClassDescription>>fileOutCategory:asHtml:"
	fileName := browser selectedClass name, '-', browser selectedMessageCategoryName, FileDirectory dot, 'st'.
	self deny: (FileDirectory default fileExists: fileName).
	
	[browser fileOutMessageCategories.
	self assert: (FileDirectory default fileExists: fileName).
	] ensure: [FileDirectory default deleteFileNamed: fileName].! !

!BrowserTest methodsFor: 'tests' stamp: 'ul 1/10/2016 22:47'!
testFlattenHierarchyTreeOnIndent
	| flatten |
	flatten := [:coll | (browser
		flattenHierarchyTree: (browser createHierarchyTreeOf: coll)
		on: OrderedCollection new
		indent: '') asArray ].
	"Empty collection."
	self assert: (flatten value: #()) isEmpty.
	"Single class."
	self assert: #('Browser') equals: (flatten value: {Browser}).
	"Single class + subclass."
	self assert: #('Browser' '  HierarchyBrowser') equals: (flatten value: {Browser. HierarchyBrowser}).
	"Single class + 2 subclasses"
	self assert: #('Browser' '  HierarchyBrowser' '  FileContentsBrowser') equals: (flatten value: { Browser. HierarchyBrowser. FileContentsBrowser. }).
	"Superclass, class, subclass"
	self assert: #('CodeHolder' '  Browser' '    HierarchyBrowser') equals: (flatten value: { CodeHolder. Browser. HierarchyBrowser. }).
	"Two 'unrelated' classes"
	self assert: #('Browser' 'SmallInteger') equals: (flatten value: { Browser. SmallInteger }).
	"Two 'unrelated' classes and a common ancestor"
	self assert: #('Browser' 'SmallInteger' 'ProtoObject') equals: (flatten value: { Browser. SmallInteger. ProtoObject }).! !

!BrowserTest methodsFor: 'tests' stamp: 'ul 1/10/2016 22:49'!
testFlattenHierarchyTreeOnIndentBy
	| flatten |
	flatten := [:coll | (browser
		flattenHierarchyTree: (browser createHierarchyTreeOf: coll)
		on: OrderedCollection new
		indent: ''
		by: 'x') asArray ].
	"Empty collection."
	self assert: (flatten value: #()) isEmpty.
	"Single class."
	self assert: #('Browser') equals: (flatten value: {Browser}).
	"Single class + subclass."
	self assert: #('Browser' 'xHierarchyBrowser') equals: (flatten value: {Browser. HierarchyBrowser}).
	"Single class + 2 subclasses"
	self assert: #('Browser' 'xHierarchyBrowser' 'xFileContentsBrowser') equals: (flatten value: { Browser. HierarchyBrowser. FileContentsBrowser. }).
	"Superclass, class, subclass"
	self assert: #('CodeHolder' 'xBrowser' 'xxHierarchyBrowser') equals: (flatten value: { CodeHolder. Browser. HierarchyBrowser. }).
	"Two 'unrelated' classes"
	self assert: #('Browser' 'SmallInteger') equals: (flatten value: { Browser. SmallInteger }).
	"Two 'unrelated' classes and a common ancestor"
	self assert: #('Browser' 'SmallInteger' 'ProtoObject') equals: (flatten value: { Browser. SmallInteger. ProtoObject }).! !

!BrowserTest methodsFor: 'tests' stamp: 'topa 4/14/2015 10:30'!
testGetSystemCategoryListIndex
	| cat |
	cat := browser systemCategoryList first.
	browser selectSystemCategory: cat.
	self assert: browser systemCategoryListIndex = (browser systemCategoryList indexOf: browser selectedSystemCategory).! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:25'!
testHasClassSelected
	browser selectSystemCategory: browser class category.
	self deny: browser hasClassSelected.

	browser selectClass: browser class.
	self assert: browser hasClassSelected.

	browser selectClass: nil.
	self deny: browser hasClassSelected.

	browser selectClassNamed: browser class name asString.
	self assert: browser hasClassSelected.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:58'!
testHasMessageCategorySelected
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser messageCategoryListIndex: 0.
	self deny: browser hasMessageCategorySelected.
	
	browser messageCategoryListIndex: 1.
	self assert: browser hasMessageCategorySelected.
	
	browser messageCategoryListIndex: 0.
	self deny: browser hasMessageCategorySelected.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:45'!
testHasMessageSelected
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser messageCategoryListIndex: 1.
	
	self deny: browser hasMessageSelected.
	
	browser messageListIndex: 0.
	self deny: browser hasMessageSelected.
	
	browser messageListIndex: 1.
	self assert: browser hasMessageSelected.
	
	browser messageListIndex: 0.
	self deny: browser hasMessageSelected.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/23/2011 17:32'!
testHomeCategory
	"This also 'just happens' to test #selectOriginalCategoryForCurrentMethod. It's not appropriate to directly test a private method. (It's a private method because its comment says so, not its category.)"
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser messageCategoryListIndex: 0. "Select -- all --"
	browser selectMessageNamed: browser messageList first.
	browser showHomeCategory.
	
	self assert: browser selectedMessageCategoryName = (browser class organization categoryOfElement: browser selectedMessageName).! !

!BrowserTest methodsFor: 'tests - message list' stamp: 'fbs 5/13/2011 18:42'!
testKeepMethodSelectedWhenChangingClass
	browser selectSystemCategory: SequenceableCollection category.
	browser selectClass: SequenceableCollection.
	browser selectMessageNamed: #add:.
	self assert: browser selectedMessageName = #add:.
	
	"Even though we have no message category selected, stay looking at the #add: method."
	browser selectClass: Collection.
	self assert: browser selectedMessageName = #add:.! !

!BrowserTest methodsFor: 'tests' stamp: 'ul 8/14/2015 18:00'!
testLabelStringAlwaysShowsBrowserType
	browser selectSystemCategory: browser class category.
	self assert: (browser labelString includesSubstring: 'System Browser').
	browser selectClass: browser class.
	self assert: (browser labelString includesSubstring: 'System Browser').
	browser selectSystemCategory: DependencyBrowser category.
	browser selectClass: DependencyBrowser.
	self assert: (browser labelString includesSubstring: 'System Browser').! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 1/3/2013 21:20'!
testLabelStringOfNewBrowserShowsBrowserType
	self assert: 'System Browser' equals: browser labelString.! !

!BrowserTest methodsFor: 'tests' stamp: 'ul 8/14/2015 18:00'!
testLabelStringShowsBrowsedClassName
	browser selectSystemCategory: DependencyBrowser category.
	browser selectClass: DependencyBrowser.
	self assert: (browser labelString includesSubstring: DependencyBrowser name).! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/5/2011 20:56'!
testMessageCatListSingleton
	| catName |
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	
	self assert: browser messageCatListSingleton = #().
	
	catName := browser messageCategoryList second.
	browser selectMessageCategoryNamed: catName.
	
	self assert: browser messageCatListSingleton = {catName}.! !

!BrowserTest methodsFor: 'tests' stamp: 'pre 9/26/2018 16:30'!
testMessageCategoryList
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	
	self assert: ({ClassOrganizer allCategory}, browser class organization categories) sorted equals: browser messageCategoryList sorted.
	
	browser selectClass: nil.
	self assert: #() equals: browser messageCategoryList.
	
	browser selectClass: browser class.
	browser metaClassIndicated: true.
	self assert: ({ClassOrganizer allCategory}, browser class classSide organization categories) equals: browser messageCategoryList.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:42'!
testMessageCategoryListIndex
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	
	self assert: browser messageCategoryList size > 1 description: 'Sanity check: test refers to first two categories.'.
	
	browser messageCategoryListIndex: 0.
	self assert: browser selectedMessageCategoryName isNil.
	
	browser messageCategoryListIndex: 1.
	self assert: browser selectedMessageCategoryName = browser messageCategoryList first.
	
	browser messageListIndex: 1.
	browser messageCategoryListIndex: 2.
	self assert: browser selectedMessageName isNil description: 'Changing category deselects method'.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/30/2011 17:34'!
testMessageList
	browser selectSystemCategory: browser class category.
	browser selectClass: nil.
	self assert: browser messageList isEmpty.
	
	browser selectClass: browser class.
	browser messageCategoryListIndex: 0.
	self assert: browser messageList = browser classOrMetaClassOrganizer allMethodSelectors.
	browser messageCategoryListIndex: 1. "-- all --"
	self assert: browser messageList = browser classOrMetaClassOrganizer allMethodSelectors.
	
	browser messageCategoryListIndex: 2.
	self assert: browser messageList = (browser classOrMetaClassOrganizer listAtCategoryNumber: 1).
	
	browser metaClassIndicated: true.
	browser messageCategoryListIndex: 0.
	self assert: browser messageList = browser classOrMetaClassOrganizer allMethodSelectors.
	
	browser messageCategoryListIndex: 1. "-- all --"
	self assert: browser messageList = browser classOrMetaClassOrganizer allMethodSelectors.
	
	browser messageCategoryListIndex: 2.
	self assert: browser messageList = (browser classOrMetaClassOrganizer listAtCategoryNumber: 1).! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/8/2011 17:08'!
testMetaClassIndicatedFalseOnExistingClass
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser metaClassIndicated: false.
	
	self assert: browser messageListIndex = 0.
	self assert: browser messageCategoryListIndex = 0.
	self deny: browser contents isEmpty. "It shows the class definition."
	
	self deny: browser messageCategoryList isEmpty.
	
	self assert: browser messageList = (browser class organization allMethodSelectors).
	self assert: browser editSelection = #editClass.! !

!BrowserTest methodsFor: 'tests' stamp: 'pre 12/11/2018 17:03'!
testMetaClassIndicatedIsResetByChangedSystemCategory
	browser selectSystemCategory: browser class category.
	browser metaClassIndicated: true.
	browser selectSystemCategory: self class category.
	
	self deny: browser metaClassIndicated.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/5/2011 20:56'!
testMetaClassIndicatedNotOnNoClass
	browser selectSystemCategory: browser class category.
	browser selectClass: nil.
	browser metaClassIndicated: false.
	
	self assert: browser messageListIndex = 0.
	self assert: browser messageCategoryListIndex = 0.
	self deny: browser contents isEmpty. "It shows the class definition."
	
	self assert: browser messageCategoryList isEmpty.
	
	self assert: browser messageList isEmpty.
	self assert: browser editSelection = #newClass.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/5/2011 20:56'!
testMetaClassIndicatedNotWithNoSystemCategorySelected
	browser selectSystemCategory: nil.
	browser metaClassIndicated: false.
	
	self assert: browser messageListIndex = 0.
	self assert: browser messageCategoryListIndex = 0.
	self assert: browser contents isEmpty. "It shows the Class definition."
	self assert: browser messageCategoryList isEmpty.
	self assert: browser editSelection = #none.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/5/2011 20:56'!
testMetaClassIndicatedOnNoClass
	browser selectSystemCategory: browser class category.
	browser selectClass: nil.
	browser metaClassIndicated: true.
	
	self assert: browser messageListIndex = 0.
	self assert: browser messageCategoryListIndex = 0.
	self assert: browser contents isEmpty.
	
	self assert: browser messageCategoryList isEmpty.
	
	self assert: browser messageList isEmpty.
	self assert: browser editSelection = #none.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/8/2011 17:08'!
testMetaClassIndicatedTrueOnExistingClass
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser metaClassIndicated: true.
	
	self assert: browser messageListIndex = 0.
	self assert: browser messageCategoryListIndex = 0.
	self deny: browser contents isEmpty. "It shows the Class definition."
	self deny: browser messageCategoryList isEmpty.
	
	self assert: browser messageList = (browser class classSide organization allMethodSelectors).! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/5/2011 20:56'!
testMetaClassIndicatedWithNoSystemCategorySelected
	browser selectSystemCategory: nil.
	browser metaClassIndicated: true.
	
	self assert: browser messageListIndex = 0.
	self assert: browser messageCategoryListIndex = 0.
	self assert: browser contents isEmpty. "It shows the Class definition."
	self assert: browser messageCategoryList isEmpty.
	self assert: browser editSelection = #none.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/5/2011 20:56'!
testRawMessageCategoryList
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	
	self assert: browser rawMessageCategoryList = browser class organization categories.
	
	browser selectClass: nil.
	self assert: browser rawMessageCategoryList = #().
	
	browser selectClass: browser class.
	browser metaClassIndicated: true.
	self assert: browser rawMessageCategoryList = browser class classSide organization categories.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/3/2011 22:30'!
testSelectCategoryForClass
	browser selectCategoryForClass: Browser.
	self assert: browser selectedSystemCategory = Browser category.
	
	browser selectCategoryForClass: TestCase.
	self assert: browser selectedSystemCategory = TestCase category.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:25'!
testSelectClass
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	self assert: browser hasClassSelected.
	
	browser selectClass: nil.
	self deny: browser hasClassSelected.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:25'!
testSelectClassEditSelectionNoClass
	browser selectSystemCategory: browser class category.
	
	browser selectClass: nil.
	self assert: browser editSelection = #newClass.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:25'!
testSelectClassEditSelectionWhenCommentSelected
	| editSelection |
	browser selectSystemCategory: Heap category.
	browser selectClass: Heap.
	browser editComment.
	browser messageListIndex: 0.
	browser messageListIndex: 0.

	editSelection := browser editSelection.
	browser selectClass: Interval.
	self assert: browser editSelection = editSelection.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:25'!
testSelectClassEditSelectionWhenEditingClass
	browser selectSystemCategory: Heap category.
	browser selectClass: Heap.
	browser metaClassIndicated: false.
	browser messageListIndex: 0.
	browser messageListIndex: 0.
	
	browser selectClass: Interval.
	self assert: browser editSelection = #editClass.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:25'!
testSelectClassEditSelectionWhenEditingMetaclass
	browser selectSystemCategory: Heap category.
	browser selectClass: Heap.
	browser metaClassIndicated: true.
	browser messageListIndex: 0.
	browser messageListIndex: 0.
	
	browser selectClass: Interval.
	self assert: browser editSelection = #editClass.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:25'!
testSelectClassEditSelectionWhenMsgSelected
	browser selectSystemCategory: Heap category.
	browser selectClass: Heap.
	browser selectMessageCategoryNamed: 'adding'.
	browser messageListIndex: (browser messageList indexOf: #add:).
	
	browser selectClass: Interval.
	self assert: browser editSelection = #editMessage.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:26'!
testSelectClassEditSelectionWhenNoMsgButCatSelected
	browser selectSystemCategory: Heap category.
	browser selectClass: Heap.
	browser selectMessageCategoryNamed: 'adding'.
	browser messageListIndex: 0.
	
	browser selectClass: Interval.
	self assert: browser editSelection = #newMessage.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/5/2011 21:16'!
testSelectClassNamed
	browser selectSystemCategory: browser class category.
	browser selectClassNamed: browser class name.
	self assert: browser selectedClassName = browser class name.
	
	browser selectClassNamed: nil.
	self assert: browser selectedClassName isNil.
	
	browser selectClassNamed: browser class name asString.
	self assert: browser selectedClassName = browser class name.! !

!BrowserTest methodsFor: 'tests' stamp: 'ul 8/14/2015 22:06'!
testSelectClassNamedPreservesPlace
	| commonCategory commonSelector otherClass |
	browser selectSystemCategory: browser class category.
	otherClass := HierarchyBrowser. "Unlikely to move out of Browser's package!!"
	browser selectClassNamed: browser class name.
	
	commonCategory := (browser class organization categories intersection: otherClass organization categories) detect: [ :each | each == #'class list' ].
	commonSelector := #classList.
	browser selectMessageCategoryNamed: commonCategory.
	browser selectMessageNamed: commonSelector.

	browser selectClass: otherClass.
	
	self assert: browser selectedClassName = otherClass name.
	self assert: browser selectedMessageCategoryName = commonCategory.
	self assert: browser selectedMessageName = commonSelector.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:26'!
testSelectClassReselectsMessage
	"Colections-Sequences contains a number of classes with similar protocols. In particular, several classes have a method called #add: in a category 'adding'."
	| selectedMsgCat selectedMethod |
	selectedMsgCat := 'adding'.
	selectedMethod := #add:.
	browser selectSystemCategory: Heap category.
	browser selectClass: Heap.
	browser selectMessageCategoryNamed: selectedMsgCat.
	browser messageListIndex: (browser messageList indexOf: selectedMethod).
	
	browser selectClass: Interval.
	
	self assert: browser selectedMessageCategoryName = selectedMsgCat.
	self assert: browser selectedMessageName = selectedMethod.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/14/2011 17:52'!
testSelectClassSelectsClass
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	self assert: browser classListIndex ~= 0.
	self assert: browser selectedClass = browser class.
	self assert: browser selectedClassName = browser class name.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/15/2011 17:40'!
testSelectClassWhenNothingSelected
	self assert: browser editSelection = #none.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 20:57'!
testSelectMessageCategoryEditSelection
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser selectMessageCategoryNamed: browser messageCategoryList first.
	
	self assert: browser editSelection = #newMessage.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 21:00'!
testSelectMessageCategoryEditSelectionNoMsgCat
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
.
	browser messageCategoryListIndex: 0.
	
	self assert: browser editSelection = #editClass.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 21:07'!
testSelectMessageCategoryEditSelectionNoMsgCatNoClass
	browser selectSystemCategory: browser class category.
	browser selectClass: nil.
.
	browser messageCategoryListIndex: 0.
	
	self assert: browser editSelection = #newClass.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/5/2011 20:57'!
testSelectMessageCategoryNamed
	| catName |
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	
	catName := browser messageCategoryList anyOne.
	browser selectMessageCategoryNamed: catName.
	self assert: browser selectedMessageCategoryName = catName.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/17/2011 07:36'!
testSelectMessageNamed
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser messageCategoryListIndex: 1.

	browser selectMessageNamed: browser messageList first.
	self assert: browser selectedMessageName = browser messageList first.
	self assert: browser messageListIndex = 1.
		
	browser selectMessageNamed: nil.
	self assert: browser selectedMessageName isNil.
	self assert: browser messageListIndex = 0.
	
	browser selectMessageNamed: browser messageList second.
	self assert: browser selectedMessageName = browser messageList second.
	self assert: browser messageListIndex = 2.
	
	browser selectMessageNamed: browser messageList first asString.
	self assert: browser selectedMessageName = browser messageList first.
	self assert: browser messageListIndex = 1.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/14/2011 21:22'!
testSelectedClass
	browser selectSystemCategory: browser class category.
	browser classListIndex: 0.
	self assert: browser classListIndex = 0.
	self assert: browser selectedClass isNil.
	
	browser classListIndex: 1.
	self assert: browser classListIndex = 1.
	self assert: browser selectedClass notNil.! !

!BrowserTest methodsFor: 'tests' stamp: 'jr 1/18/2017 12:47'!
testSelectedEnvironment
	browser selectSystemCategory: browser class category.
	self assert: browser selectedEnvironment = Smalltalk globals.
	
	"browser selectSystemCategory: nil.
	self assert: browser selectedEnvironment isNil."
	"selectedClass would not work in a hierarchy browser that has to display classes from different environments (because the correct categories might be missing)"! !

!BrowserTest methodsFor: 'tests' stamp: 'topa 4/14/2015 10:28'!
testSelectedSystemCategory
	| cat newCat |
	cat := browser systemCategoryList first.
	newCat := browser systemCategoryList second.
	browser selectSystemCategory: cat.
	self assert:	browser selectedSystemCategory = cat.
		
	browser systemCategoryListIndex: 2.
	self assert:	browser selectedSystemCategory = newCat.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/17/2011 17:58'!
testShowHomeCategory
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	"Select '--all--'."
	browser selectMessageCategoryNamed: browser messageCategoryList first.
	browser selectMessageNamed: browser messageList first.
	
	browser showHomeCategory.
	self assert: browser messageCategoryListIndex > 1.! !

!BrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:26'!
testSpawnHierarchy
	| spawn |
	browser selectSystemCategory: browser class category.
	browser selectClass: browser class.
	browser selectMessageCategoryNamed: browser messageCategoryList last.
	browser messageListIndex: (browser messageList size).

	spawn := browser spawnHierarchy.
	[self assert: spawn selectedSystemCategory = browser selectedSystemCategory.
	self assert: spawn selectedClassName = browser selectedClassName.
	self assert: spawn selectedMessageCategoryName = browser selectedMessageCategoryName.
	self assert: spawn messageListIndex = browser messageListIndex.]
		ensure: [ ToolBuilder default close: spawn topView ].! !

!BrowserTest methodsFor: 'tests' stamp: 'topa 4/14/2015 10:28'!
testSystemCategoryListIndex
	| oldCat |
	
	browser systemCategoryListIndex: 1.
	oldCat := browser selectedSystemCategory.
	self assert:	browser selectedSystemCategory = browser systemCategoryList first.
		
	browser systemCategoryListIndex: 2.
	self deny: browser selectedSystemCategory = oldCat.
	self assert:	browser selectedSystemCategory = browser systemCategoryList second.! !

TestCase subclass: #DebuggerExtensionsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Debugger'!

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testBooleansGeneralise
	self assert: Boolean name equals: true canonicalArgumentName.
	self assert: Boolean name equals: false canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testByteStringsGeneralise
	self assert: String name equals: ByteString new canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testClassesGeneralise
	self assert: Class name equals: Class new canonicalArgumentName.
	self assert: Class name equals: 1 class canonicalArgumentName.
	self assert: Class name equals: true class canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: 'dtl 5/5/2018 10:27'!
testCollectionsGeneralise
	"ArrayedCollection and subclasses answer 'Array' "
	self assert: Array name equals: Array new canonicalArgumentName.
	"Otherwise answer the more general 'Collection' "
	self assert: Collection name equals: OrderedCollection new canonicalArgumentName.
	self assert: Collection name equals: LinkedList new canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testFloatsGeneralise
	self assert: Number name equals: 1.0 canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testFractionsGeneralise
	self assert: Number name equals: (1 / 2) canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testIntegersDontGeneralise
	self assert: Integer name equals: 1 canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testLargeNegativeIntegersGeneralise
	self assert: Integer name equals: LargeNegativeInteger initializedInstance canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testLargePositiveIntegersGeneralise
	self assert: Integer name equals: LargePositiveInteger initializedInstance canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testOtherObjectsDontGeneralise
	self assert: Behavior name equals: Behavior new canonicalArgumentName.
	self assert: UndefinedObject name equals: nil canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testScaledDecimalsGeneralise
	self assert: Number name equals: 2.00s2 canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testStringsDontGeneralise
	self assert: String name equals: String new canonicalArgumentName! !

!DebuggerExtensionsTest methodsFor: 'tests - canonical argument names' stamp: ''!
testWideStringsGeneralise
	self assert: String name equals: WideString new canonicalArgumentName! !

TestCase subclass: #DebuggerTests
	instanceVariableNames: 'process debugger window reset'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Debugger'!

!DebuggerTests commentStamp: 'mt 10/1/2019 14:24' prior: 0!
You must be in a Morphic project to run these tests!! Note that *ALL* tests construct new processes and leave the current UI process as is. Even errors are injected in helper processes. So, the test runner will not suspend or terminate when running these tests.

Note that many of these tests depend on the StandardToolSet. So, if you have a custom tool set, maybe reset it first.!

!DebuggerTests methodsFor: 'support' stamp: 'mt 10/1/2019 13:54'!
doTestUserInterruptFor: block priority: priority
	"Note that we cannot signal the InterruptSemaphore because there is already the regular process waiting for it. We simulate that semaphore with a custom delay."
	
	| work wait interruptHelper |
	work := Semaphore new.
	wait := Semaphore new. "Maybe not necessary?"
	
	process := [ work wait. block value ] newProcess.
	process priority: priority.
	self assert: process priority >= Processor activeProcess priority. "You cannot interrupt lower priority processes."
	process resume.
	
	interruptHelper := [ wait wait. 10 milliSeconds wait. Smalltalk handleUserInterrupt ] newProcess.
	interruptHelper priority: Processor lowIOPriority.
	self assert: interruptHelper priority > process priority.
	interruptHelper resume.
	
	[wait signal. work signal] valueNoContextSwitch.
	process priority = Processor activeProcess priority ifTrue: [Processor yield].
	
	self assert: interruptHelper isTerminated.

	self ensureDebugger.! !

!DebuggerTests methodsFor: 'support' stamp: 'mt 10/1/2019 13:38'!
ensureDebugger

	debugger ifNotNil: [debugger close].
	
	debugger := nil.
	window := nil.
	
	self assert: process notNil.
	self assert: process isSuspended.

	self findDebugger.	
	
	self assert: window notNil.
	self assert: debugger notNil.! !

!DebuggerTests methodsFor: 'support' stamp: 'mt 10/1/2019 13:25'!
findDebugger

	window := nil.
	debugger := nil.
		
	self findDebuggerWindowFor: process.

	window ifNil: [
		self updateUserInterface.
		self findDebuggerWindowFor: process].

	window ifNotNil: [debugger := window model]! !

!DebuggerTests methodsFor: 'support' stamp: 'mt 10/1/2019 11:23'!
findDebuggerWindowFor: process

	window :=  (Project current world in: [:world | world submorphs, world firstHand submorphs])
		detect: [:m | (m isSystemWindow and: [m model isKindOf: Debugger]) and: [m model interruptedProcess == process]]
		ifNone: [].! !

!DebuggerTests methodsFor: 'support' stamp: 'mt 10/6/2019 13:55'!
installToolSet

	ToolSet default: DebuggerTestsToolSet new.! !

!DebuggerTests methodsFor: 'running' stamp: 'mt 10/15/2019 09:10'!
setUp

	super setUp.
	
	Project current isMorphic ifFalse: [self fail].
	Project current uiProcess isActiveProcess ifFalse: [self fail].

	reset := {
		[:enable | [enable ifTrue: [Preferences enable: #logDebuggerStackToFile]]]
			value: Preferences logDebuggerStackToFile.
		[:ts | [self uninstallToolSet. ToolSet default: ts]]
			value: ToolSet default.
	}.
	
	Preferences disable: #logDebuggerStackToFile.! !

!DebuggerTests methodsFor: 'running' stamp: 'mt 10/5/2019 10:20'!
tearDown

	debugger ifNotNil: [debugger close].
	process ifNotNil: [process terminate].
	
	process := nil.
	debugger := nil.
	window := nil.

	reset do: #value.	
	
	super tearDown.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/7/2019 09:41'!
test01UserInterrupt
	
	| counter |
	{
		[ [counter := counter + 1] repeat ]. 
		[ [counter := counter + 1. true] whileTrue ]. 
		"[ counter := counter + 1. Smalltalk createStackOverflow ]." "Disabled because VM may actually crash under certain conditions."
	} do: [:workBlock |
		{
			Processor userSchedulingPriority.
			Processor userSchedulingPriority + 1.
			Processor userInterruptPriority
		} do: [:workPriority |
			counter := 0.
			self doTestUserInterruptFor: workBlock priority: workPriority.
			self assert: counter > 0.
			self assert: debugger isNotifier]].! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 14:14'!
test02UnhandledException

	| counter |
	{
		[ counter := counter + 1. 7/0 ]. 'ZeroDivide'. 
		[ counter := counter + 1. Halt signal ]. 'Halt'.
		[ counter := counter + 1. Warning signal: 'Test' ]. 'Warning'
	} pairsDo: [:errorBlock :label |
		Processor systemBackgroundPriority
			to: Processor timingPriority
			by: 10
			do: [:priority |
				counter := 0.
	
				process := errorBlock forkAt: priority.
	
				"Let lower priority processes run to raise their exception."
				priority <= Processor activeProcess priority ifTrue: [
					3 timesRepeat: [process isSuspended not ifTrue: [50 milliSeconds wait]]].
				
				self ensureDebugger.
				
				self assert: counter > 0.
				self assert: (window label beginsWith: label).
				self assert: debugger isNotifier]]! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 13:38'!
test03AsyncronousExceptions
	"We want to get n different debuggers from n different processes."
	
	((1 to: 3) "... more than 1 ... so ... 3 sounds about right ..."
		collect: [:inc | [ Error signal ] forkAt: Processor userSchedulingPriority + inc])
		do: [:errorProcess |
				"There must be debuggers for all processes."
				process := errorProcess.
				self ensureDebugger];
		in: [:all | debugger close "Just close the last debugger."];
		do: [:errorProcess |
				process := errorProcess.
				self findDebugger.
				
				self assert: debugger isNil.
				self assert: process isTerminated]! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 14:58'!
test04DebuggerSuspendsProcess
	"Opening the debugger on a running process means to suspend that process."

	process := [ [3 + 4] repeat ] newProcess.
	
	"Choose a priority lower than the active process so that we can use an endless loop."
	process priority: Processor activeProcess priority - 1.
	process resume. Processor yield.

	self deny: process isSuspended.

	debugger := process debug.
	
	self assert: process isSuspended.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 14:58'!
test05DebuggerTerminatesProcess
	"Closing a debugger on a suspended process means terminating that process."
	
	process := [ [3 + 4] repeat ] newProcess.
	
	"Choose a priority lower than the active process so that we can use an endless loop."
	process priority: Processor activeProcess priority - 1.
	process resume. Processor yield.

	self deny: process isTerminated.

	debugger := process debug.
	debugger close.
	
	self assert: process isTerminated.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 14:58'!
test06DebugSpecificContext
	
	| context |

	process := [ #(1 2 3) collect: [:ea | ea odd ifTrue: [ ea ] ifFalse: [ Processor activeProcess suspend ] ] ] newProcess.
	process priority: Processor activeProcess priority + 1.
	process resume.

	"Find specific context by following #sender chain."
	context := process suspendedContext.
	[context selector = #collect: ] whileFalse: [context := context sender].

	debugger := process debug: context.

	self assert: debugger contextStackTop = context.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 14:57'!
test07DebuggerNotifier
	
	process := [ 3+4 ] newProcess.
	debugger := process debugWithTitle: 'Test' full: false.
	self assert: debugger isNotifier.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 14:57'!
test08DebuggerFull
	
	process := [ 3+4 ] newProcess.
	debugger := process debugWithTitle: 'Test' full: true.
	self assert: debugger isFull.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 14:57'!
test09DebuggerNotifierOrFull
	"Test the defaults."
		
	process := [ 3+4 ] newProcess.
	debugger := process debug.
	self assert: debugger isFull.
	
	debugger close.
	
	process := [ 3+4 ] newProcess.
	debugger := process debugWithTitle: 'Test'.
	self assert: debugger isFull.

	debugger close.
	
	process := [ 3+4 ] newProcess.
	debugger := process debug: process suspendedContext.
	self assert: debugger isNotifier.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/1/2019 17:38'!
test10DebugBlock
	"Used for debug-it in tools."

	process := Process forBlock: [3+4].
	debugger := process debug.

	self deny: 7 equals: process suspendedContext top.
	debugger stepOver.
	self assert: 7 equals: process suspendedContext top.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/2/2019 10:13'!
test11DebugBlockAtContext
	"Similar to the run-to feature in the debugger, run the process until a certain condition is met."
	
	process := Process
		forBlock: [(#(1 2 3) collect: [:ea | ea + 1]) in: [:all | all sum]]
		runUntil: [:context | context selector = #sum].
	debugger := process debug.

	self deny: 9 equals: process suspendedContext top.
	debugger stepOver.
	self assert: 9 equals: process suspendedContext top.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/6/2019 14:00'!
test12ToolSetHandleError
	"Test whether the #handle* callback is reached as expected."
	
	self installToolSet.

	process := [ Error signal ] newProcess.
	process priority: Processor userSchedulingPriority + 1.
	
	self assert: 0 equals: self toolSet handledErrors size.
	process resume.
	self assert: 1 equals: self toolSet handledErrors size.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/6/2019 14:01'!
test13ToolSetHandleWarning
	"Test whether the #handle* callback is reached as expected."
	
	self installToolSet.

	process := [ Warning signal ] newProcess.
	process priority: Processor userSchedulingPriority + 1.
	
	self assert: 0 equals: self toolSet handledWarnings size.
	process resume.
	self assert: 1 equals: self toolSet handledWarnings size.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/6/2019 14:07'!
test14ToolSetHandleUserInterruptRequest
	"Test whether the #handle* callback is reached as expected."
	
	self installToolSet.

	process := [ Smalltalk handleUserInterrupt ] newProcess.
	process priority: Processor userSchedulingPriority + 1.
	
	self assert: 0 equals: self toolSet handledUserInterruptRequests size.
	process resume.
	self assert: 1 equals: self toolSet handledUserInterruptRequests size.! !

!DebuggerTests methodsFor: 'tests' stamp: 'mt 10/6/2019 17:50'!
test15ToolSetDebugProcess
	"Test whether the #debug* callback is reached as expected."
	
	self installToolSet.

	process := [ 3+4 ] newProcess.
	
	self assert: 0 equals: self toolSet debuggedProcesses size.
	process debug.
	self assert: 1 equals: self toolSet debuggedProcesses size.! !

!DebuggerTests methodsFor: 'support' stamp: 'mt 10/6/2019 13:55'!
toolSet

	^ ToolSet default! !

!DebuggerTests methodsFor: 'support' stamp: 'mt 10/15/2019 09:09'!
uninstallToolSet

	(ToolSet default isKindOf: DebuggerTestsToolSet) ifTrue: [
		ToolSet unregister: ToolSet default].! !

!DebuggerTests methodsFor: 'support' stamp: 'mt 10/1/2019 10:17'!
updateUserInterface

	Project current world doOneCycle.! !

TestCase subclass: #DebuggerUnwindBug
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Debugger'!

!DebuggerUnwindBug methodsFor: 'tests' stamp: 'ar 3/7/2003 01:38'!
testUnwindBlock
	"test if unwind blocks work properly"
	| sema process |
	sema := Semaphore forMutualExclusion.
	self assert: sema isSignaled.
	"deadlock on the semaphore"
	process := [sema critical:[sema wait]] forkAt: Processor userInterruptPriority.
	self deny: sema isSignaled.
	"terminate process"
	process terminate.
	self assert: sema isSignaled.
! !

!DebuggerUnwindBug methodsFor: 'tests' stamp: 'mt 9/30/2019 17:31'!
testUnwindDebugger
	"test if unwind blocks work properly when a debugger is closed"
	| sema process debugger top |
	sema := Semaphore forMutualExclusion.
	self assert: sema isSignaled.
	process := [sema critical:[sema wait]] forkAt: Processor userInterruptPriority.
	self deny: sema isSignaled.

	"everything set up here - open a debug notifier"
	debugger := process debugWithTitle: 'test' full: false.
	"get into the debugger"
	debugger debug.
	top := debugger topView.
	"set top context"
	debugger toggleContextStackIndex: 1.
	"close debugger"
	top delete.

	"and see if unwind protection worked"
	self assert: sema isSignaled.! !

TestCase subclass: #DependencyBrowserTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!DependencyBrowserTest methodsFor: 'running' stamp: 'fbs 5/4/2011 18:43'!
setUp
	browser := DependencyBrowser new.! !

!DependencyBrowserTest methodsFor: 'tests - buttons' stamp: 'fbs 5/5/2011 17:55'!
testBrowse
	| b |
	browser packageListIndex: (browser packageList indexOf: #Collections).
	browser packageDepsIndex: (browser packageDeps indexOf: #Compiler).
	browser classDepsIndex: (browser classDeps indexOf: #Scanner).
	browser classListIndex: (browser classList indexOf: #Symbol).
	browser messageListIndex: 1.

	b := browser browseMethodFull.
	self assert: b notNil.
	[self assert: b selectedMessageName = (browser messageList at: browser messageListIndex) ]
		ensure: [ToolBuilder default close: b topView]! !

!DependencyBrowserTest methodsFor: 'tests - class list' stamp: 'pre 10/12/2018 14:07'!
testClassList
	"Warning!! When Collections' dependencies change, this test may start to fail!!"
	self assert: #() equals: browser classList description: 'classList before package selection'.
	
	browser packageListIndex: (browser packageList indexOf: #Collections).
	self 
		assert: #()
		equals: browser classList 
		description: 'classList after selecting Collections'.
	
	browser packageDepsIndex: (browser packageDeps indexOf: #Compiler).
	self 
		assert: #() 
		equals: browser classList 
		description: 'classList after selecting Compiler'.
	
	browser classDepsIndex: (browser classDeps indexOf: #Scanner).
	self 
		assert: #(Character String Symbol 'Symbol class') 
		equals: browser classList 
		description: 'classList after selecting Scanner class'.
	
	browser classListIndex: (browser classList indexOf: #Symbol).
	self 
		assert: #(Character String Symbol 'Symbol class') 
		equals: browser classList 
		description: 'classList after selecting Symbol class'.! !

!DependencyBrowserTest methodsFor: 'tests - metaclass' stamp: 'fbs 5/5/2011 17:53'!
testHasPackageSelected
	self deny: browser hasPackageSelected.
	
	browser packageListIndex: 1.
	self assert: browser hasPackageSelected.
	
	browser packageListIndex: 0.
	self deny: browser hasPackageSelected.! !

!DependencyBrowserTest methodsFor: 'tests - metaclass' stamp: 'fbs 5/6/2011 07:43'!
testSelectedClassOrMetaClass
	"Warning!! When Collections' dependencies change, this test may start to fail!!"
	self assert: browser selectedClassOrMetaClass isNil.

	browser packageListIndex: (browser packageList indexOf: #Collections).
	self assert: browser selectedClassOrMetaClass isNil.
	
	browser packageDepsIndex: (browser packageDeps indexOf: #Compiler).
	self assert: browser selectedClassOrMetaClass isNil.
	
	browser classDepsIndex: (browser classDeps indexOf: #Scanner).
	self assert: browser selectedClassOrMetaClass isNil.
	
	browser classListIndex: (browser classList indexOf: #Symbol).

	self deny: browser selectedClassOrMetaClass isNil.
	self assert: browser selectedClassOrMetaClass == Symbol.! !

!DependencyBrowserTest methodsFor: 'tests - message list' stamp: 'pre 5/15/2019 21:24'!
testSelectedMessageClassSideShowsSource
	"Warning!! When Collections' dependencies change, this test may start to fail!!"

	browser packageListIndex: (browser packageList indexOf: #'Chronology-Core').
	browser packageDepsIndex: (browser packageDeps indexOf: #Collections).
	browser classDepsIndex: (browser classDeps indexOf: #'Array').
	browser classListIndex: (browser classList indexOf: #'DateAndTime').
	browser messageListIndex: 1.
	self assert: browser selectedMessageName == #ticks.
	
	self assert: browser contents = (DateAndTime sourceCodeAt: #ticks)
! !

!DependencyBrowserTest methodsFor: 'tests - message list' stamp: 'fbs 5/6/2011 08:26'!
testSelectedMessageName
	"Warning!! When Collections' dependencies change, this test may start to fail!!"
	self assert: browser selectedMessageName isNil.

	browser packageListIndex: (browser packageList indexOf: #Collections).
	self assert: browser selectedMessageName isNil.
	
	browser packageDepsIndex: (browser packageDeps indexOf: #Compiler).
	self assert: browser selectedMessageName isNil.
	
	browser classDepsIndex: (browser classDeps indexOf: #Scanner).
	self assert: browser selectedMessageName isNil.
	
	browser classListIndex: (browser classList indexOf: #Symbol).
	self assert: browser selectedMessageName isNil.
	
	browser messageListIndex: 1.
	self assert: browser selectedMessageName == #storeOn:.
	
	browser messageListIndex: 0.
	self assert: browser selectedMessageName isNil.! !

!DependencyBrowserTest methodsFor: 'tests - message list' stamp: 'pre 5/15/2019 21:24'!
testSelectedMessageNameClassSide
	"Warning!! When Collections' dependencies change, this test may start to fail!!"
	self assert: browser selectedMessageName isNil.

	browser packageListIndex: (browser packageList indexOf: #'Chronology-Core').
	self assert: browser selectedMessageName isNil.
	
	browser packageDepsIndex: (browser packageDeps indexOf: #Collections).
	self assert: browser selectedMessageName isNil.
	
	browser classDepsIndex: (browser classDeps indexOf: #Array).
	self assert: browser selectedMessageName isNil.
	
	browser classListIndex: (browser classList indexOf: #'DateAndTime').
	self assert: browser selectedMessageName isNil.
	
	browser messageListIndex: 1.
	self assert: browser selectedMessageName == #ticks.
! !

TestCase subclass: #FileContentsBrowserTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!FileContentsBrowserTest methodsFor: 'running' stamp: 'pre 12/1/2018 17:20'!
setUp
	| fileout testCat |
	testCat := self class category.
	fileout := WriteStream on: (String new: 10000).
	SystemOrganization fileOutCategory: testCat on: fileout.
	browser := FileContentsBrowser createBrowserForStream: fileout readStream named: testCat.! !

!FileContentsBrowserTest methodsFor: 'tests' stamp: 'pre 12/27/2018 21:49'!
testBrowsingAChangesetShowsMethods

	| newChangeSet fileContents internalStream class |
	newChangeSet := ChangeSet basicNewNamed: 'fooTestSet' , (Random new nextLargeInt: 10000) asString.
	SystemChangeNotifier uniqueInstance doSilently: [
		class := Object subclass: #FileContentsBrowserTestTestObject
			instanceVariableNames: ''
			classVariableNames: ''
			poolDictionaries: ''
			category: self class category name asString.
		class compileSilently: 'theTestMethod ^ false'].
	
	newChangeSet adoptSelector: #theTestMethod forClass: class.

	internalStream := WriteStream on: (String new: 10000).
	newChangeSet fileOutOn: internalStream.
	fileContents := internalStream contents.
	class removeFromSystem.
	
	browser := FileContentsBrowser createBrowserForStream: (MultiByteBinaryOrTextStream with: fileContents) named: 'testFoo'.

	browser selectSystemCategory: browser systemCategoryList first.
	browser selectClassNamed: browser classList first.
	
	self assert: browser messageList size = 1.! !

!FileContentsBrowserTest methodsFor: 'tests' stamp: 'pre 12/27/2018 21:41'!
testChangeMessageCategoriesLeavesClassSelected
	browser selectSystemCategory: browser systemCategoryList first.
	browser selectClassNamed: self class name.
	
	"This is a no-op."
	browser changeMessageCategories: browser classOrMetaClassOrganizer printString.
	self assert: browser selectedClassName = self class name.! !

TestCase subclass: #FileList2ModalDialogsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-FileList'!

!FileList2ModalDialogsTest commentStamp: '<historical>' prior: 0!
TestRunner open!

!FileList2ModalDialogsTest methodsFor: 'tests' stamp: 'LC 1/6/2002 08:45'!
testModalFileSelector
	| window fileList2 |
	window := FileList2 morphicViewFileSelector.
	window openCenteredInWorld.
	fileList2 := window valueOfProperty: #fileListModel.
	fileList2 fileListIndex: 1.
	window delete.
	self assert: fileList2 getSelectedFile isNil.
	fileList2 okHit.
	self deny: fileList2 getSelectedFile isNil


! !

!FileList2ModalDialogsTest methodsFor: 'tests' stamp: 'LC 1/6/2002 08:50'!
testModalFileSelectorForSuffixes
	| window fileList2 |
	window := FileList2 morphicViewFileSelectorForSuffixes: nil.
	window openCenteredInWorld.
	fileList2 := window valueOfProperty: #fileListModel.
	fileList2 fileListIndex: 1.
	window delete.
	self assert: fileList2 getSelectedFile isNil.
	fileList2 okHit.
	self deny: fileList2 getSelectedFile isNil
! !

!FileList2ModalDialogsTest methodsFor: 'tests' stamp: 'LC 1/6/2002 08:55'!
testModalFolderSelector
	| window fileList2 |
	window := FileList2 morphicViewFolderSelector.
	fileList2 := window model.
	window openInWorld: self currentWorld extent: 300@400.
	fileList2 fileListIndex: 1.
	window delete.
	self assert: fileList2 getSelectedDirectory withoutListWrapper isNil.
	fileList2 okHit.
	self deny: fileList2 getSelectedDirectory withoutListWrapper isNil

! !

!FileList2ModalDialogsTest methodsFor: 'tests' stamp: 'LC 1/6/2002 09:01'!
testModalFolderSelectorForProjectLoad
	| window fileList2 w |
	window := FileList2
		morphicViewProjectLoader2InWorld: self currentWorld
		reallyLoad: false.
	fileList2 := window valueOfProperty: #FileList.
	w := self currentWorld.
	window position: w topLeft + (w extent - window extent // 2).
	window openInWorld: w.
	window delete.
	self assert: fileList2 getSelectedDirectory withoutListWrapper isNil.
	fileList2 okHit.
	self deny: fileList2 getSelectedDirectory withoutListWrapper isNil
! !

TestCase subclass: #HierarchyBrowserTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!HierarchyBrowserTest methodsFor: 'running' stamp: 'fbs 3/5/2011 22:22'!
setUp
	browser := HierarchyBrowser new.! !

!HierarchyBrowserTest methodsFor: 'tests' stamp: 'fbs 3/12/2011 20:48'!
testClassList
	| c displayedClasses hierarchy |
	browser initHierarchyForClass: browser class.
	
	c := browser class theNonMetaClass.
	displayedClasses := browser classList collect: [:each | each withoutLeadingBlanks ].
	hierarchy := (c allSuperclasses reversed, {browser class}, c allSubclasses) collect: [ :each | each name ].
	self assert: displayedClasses = hierarchy.! !

!HierarchyBrowserTest methodsFor: 'tests' stamp: 'fbs 3/14/2011 07:35'!
testClassListIndex
	| expectedCat |
	expectedCat := [:i | (Smalltalk at: (browser classList at: i) withoutLeadingBlanks asSymbol) category ].
	browser initHierarchyForClass: browser class.
	self deny: browser classListIndex = 0.
	
	1 to: 3 do:
		[:n |
		browser classListIndex: n.
		self assert: browser classListIndex = n.
		self assert: browser selectedSystemCategory = (expectedCat value: n)]! !

!HierarchyBrowserTest methodsFor: 'tests' stamp: 'fbs 3/12/2011 21:08'!
testClassListSansWhitespace
	| c displayedClasses hierarchy |
	browser initHierarchyForClass: browser class.
	
	c := browser class theNonMetaClass.
	displayedClasses := browser classList collect: [:each | each withoutLeadingBlanks ].
	hierarchy := (c allSuperclasses reversed, {browser class}, c allSubclasses) collect: [ :each | each name ].
	self assert: displayedClasses = hierarchy.! !

!HierarchyBrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 10:12'!
testInitHierarchyForClass
	| b |

	b := HierarchyBrowser new initHierarchyForClass: browser class.
	self assert: b selectedClassName = browser class name.! !

!HierarchyBrowserTest methodsFor: 'tests' stamp: 'fbs 3/14/2011 17:30'!
testSelectClass
	browser initHierarchyForClass: browser class.
	browser selectClass: ProtoObject.
	self assert: browser selectedClass = ProtoObject.
	
	browser selectClass: CodeHolder.
	self assert: browser selectedClass = CodeHolder.! !

!HierarchyBrowserTest methodsFor: 'tests' stamp: 'fbs 3/12/2011 20:48'!
testSelectedClassName
	browser initHierarchyForClass: browser class.
	self assert: browser selectedClassName = browser class name.
	
	browser classListIndex: 0.
	self assert: browser selectedClassName isNil.! !

TestCase subclass: #MessageNamesTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!MessageNamesTest methodsFor: 'running' stamp: 'fbs 3/23/2011 21:40'!
setUp
	browser := MessageNames openMessageNames.! !

!MessageNamesTest methodsFor: 'running' stamp: 'mt 4/3/2015 22:20'!
tearDown
	browser topView ifNotNil: [:v |
		ToolBuilder default close: v].! !

!MessageNamesTest methodsFor: 'tests' stamp: 'fbs 3/24/2011 07:36'!
testCannotUnselectCategory
	| selectorList |
	browser searchString: 'cull'.
	browser selectorListIndex: 1.
	selectorList := browser selectorList copy.
	
	browser selectorListIndex: 0.
	self assert: selectorList = browser selectorList.! !

!MessageNamesTest methodsFor: 'tests' stamp: 'fbs 3/23/2011 21:40'!
testOpenMessageNames
	| b |
	b := MessageNames openMessageNames.
	[self assert: (b isKindOf: MessageNames)]
		ensure: [ToolBuilder default close: b topView].! !

!MessageNamesTest methodsFor: 'tests' stamp: 'fbs 3/23/2011 21:48'!
testSearchString
	| cCount cullCount cullCullCount |
	browser searchString: 'c'.
	cCount := browser selectorList size.
	
	browser searchString: 'cull:'.
	cullCount := browser selectorList size.
	
	self assert: cullCount < cCount.
	
	browser searchString: 'cull:cull:'.
	cullCullCount := browser selectorList size.
	
	self assert: cullCullCount < cullCount.! !

!MessageNamesTest methodsFor: 'tests' stamp: 'pre 12/11/2018 17:00'!
testSelectedClassOrMetaClass
	| firstClass |
	browser searchString: 'cull:'.
	firstClass := browser messageList first actualClass.
	
	browser messageListIndex: 1.
	self assert: browser selectedClassOrMetaClass = firstClass.
	
	browser messageListIndex: 0.
	self assert: browser selectedClassOrMetaClass isNil.
	
	"And just to be sure:"
	browser messageListIndex: 1.
	self assert: browser selectedClassOrMetaClass = firstClass.! !

!MessageNamesTest methodsFor: 'tests' stamp: 'pre 12/11/2018 17:01'!
testSelectedMessageCategoryName
	browser searchString: 'cull:'.
	browser selectorListIndex: 1.
	browser messageListIndex: 1.
	self deny: browser selectedMessageCategoryName isNil.

	browser messageListIndex: 0.
	self assert: browser selectedMessageCategoryName isNil.! !

!MessageNamesTest methodsFor: 'tests' stamp: 'pre 12/11/2018 17:00'!
testSelection
	browser searchString: 'cull:'.
	browser selectorListIndex: 1.
	browser messageListIndex: 1.
	self assert: (browser selection isKindOf: MethodReference).
	browser messageListIndex: 0.
	self assert: browser selection isNil.! !

TestCase subclass: #MessageSetTest
	instanceVariableNames: 'browser updates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!MessageSetTest methodsFor: 'initialize-release' stamp: 'fbs 4/17/2011 22:23'!
initialize
	super initialize.
	
	"Contains the triggers sent by the model during its updates."
	updates := OrderedCollection new.! !

!MessageSetTest methodsFor: 'running' stamp: 'fbs 4/17/2011 22:23'!
setUp
	browser := (MessageSet
		openMessageList: (SystemNavigation default allImplementorsOf: #browseAllImplementorsOf:)
		name: 'Test MessageSet'
		autoSelect: nil) model.

	browser addDependent: self.! !

!MessageSetTest methodsFor: 'running' stamp: 'fbs 3/22/2011 17:34'!
tearDown
	ToolBuilder default close: browser topView.! !

!MessageSetTest methodsFor: 'tests' stamp: 'fbs 4/17/2011 22:22'!
testAutoSelectString
	browser autoSelectString: 'browseAllImplementorsOf:'.
	self assert: browser autoSelectString = 'browseAllImplementorsOf:'.
	browser messageListIndex: 2.
	self assert: (updates includes: #autoSelect).! !

!MessageSetTest methodsFor: 'tests' stamp: 'fbs 3/29/2011 16:56'!
testContents
	browser messageListIndex: 0.
	self assert: browser contents = ''.
	
	browser messageListIndex: 1.
	self deny: browser contents = ''.
	! !

!MessageSetTest methodsFor: 'tests' stamp: 'fbs 3/29/2011 16:56'!
testHasMessageSelected
	browser messageListIndex: 1.
	self assert: browser hasMessageSelected.
	
	browser messageListIndex: 0.
	self deny: browser hasMessageSelected.
	
	"And just to be sure:"
	browser messageListIndex: 1.
	self assert: browser hasMessageSelected.! !

!MessageSetTest methodsFor: 'tests' stamp: 'fbs 3/29/2011 16:56'!
testRemoveMessageFromBrowser
	| count sel |
	browser messageListIndex: 1.
	sel := browser selection.
	
	browser removeMessageFromBrowser.
	self deny: (browser messageList includes: sel) description: 'Method not removed'.

	browser messageListIndex: 0.
	count := browser messageList size.
	browser removeMessageFromBrowser.
	self assert: count = browser messageList size description: 'Some method was removed'.! !

!MessageSetTest methodsFor: 'tests' stamp: 'fbs 3/29/2011 16:57'!
testSelectedClassOrMetaClass
	| firstClass |
	firstClass := browser messageList first actualClass.
	
	browser messageListIndex: 1.
	self assert: browser selectedClassOrMetaClass = firstClass.
	
	browser messageListIndex: 0.
	self assert: browser selectedClassOrMetaClass isNil.
	
	"And just to be sure:"
	browser messageListIndex: 1.
	self assert: browser selectedClassOrMetaClass = firstClass.! !

!MessageSetTest methodsFor: 'tests' stamp: 'fbs 3/29/2011 16:57'!
testSelectedMessageCategoryName
	| firstCat methodRef |
	methodRef := browser messageList first.
	firstCat := methodRef actualClass organization categoryOfElement: methodRef selector.
	
	browser messageListIndex: 1.
	self assert: browser selectedMessageCategoryName = firstCat.
	
	browser messageListIndex: 0.
	self assert: browser selectedMessageCategoryName isNil.
	
	"And just to be sure:"
	browser messageListIndex: 1.
	self assert: browser selectedMessageCategoryName = firstCat.! !

!MessageSetTest methodsFor: 'tests' stamp: 'fbs 3/29/2011 16:58'!
testSelectedMessageName
	| firstSel |
	firstSel := browser messageList first selector.
	
	browser messageListIndex: 1.
	self assert: browser selectedMessageName = firstSel.
	
	browser messageListIndex: 0.
	self assert: browser selectedMessageName isNil.
	
	"And just to be sure:"
	browser messageListIndex: 1.
	self assert: browser selectedMessageName = firstSel.! !

!MessageSetTest methodsFor: 'tests' stamp: 'fbs 3/29/2011 16:57'!
testSelection
	| firstRef |
	firstRef := browser messageList first.
	
	browser messageListIndex: 1.
	self assert: browser selection = firstRef.
	
	browser messageListIndex: 0.
	self assert: browser selection isNil.
	
	"And just to be sure:"
	browser messageListIndex: 1.
	self assert: browser selection = firstRef.! !

!MessageSetTest methodsFor: 'private' stamp: 'fbs 4/17/2011 22:28'!
update: aSelector
	updates addLast: aSelector.! !

MessageSetTest subclass: #MessageTraceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!MessageTraceTest methodsFor: 'running' stamp: 'fbs 4/17/2011 22:28'!
setUp
	browser := (MessageTrace
		openMessageList: (SystemNavigation default allImplementorsOf: #browseAllImplementorsOf:)
		name: 'Test MessageSet'
		autoSelect: nil) model.
		
	browser addDependent: self.! !

!MessageTraceTest methodsFor: 'tests' stamp: 'fbs 5/21/2013 23:14'!
testRemoveMessageFromBrowser
	| count sel |
	browser messageListIndex: 1.
	sel := browser selection.
	
	browser removeMessageFromBrowser.
	self deny: (browser messageList includes: sel) description: 'Method not removed'.

	"In a MessageTrace, having an unselected message and then removing it means 'remove the first message'."
	browser messageListIndex: 0.
	count := browser messageList size.
	browser removeMessageFromBrowser.
	self assert: count - 1 = browser messageList size description: 'Method not removed'.! !

TestCase subclass: #PackagePaneBrowserTest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Browser'!

!PackagePaneBrowserTest methodsFor: 'running' stamp: 'fbs 3/3/2011 23:04'!
setUp
	browser := PackagePaneBrowser new.! !

!PackagePaneBrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 17:26'!
testClassList
	browser selectSystemCategory: browser class category.
	self assert: browser classList = (SystemOrganization listAtCategoryNamed: browser class category).
	
	browser selectSystemCategory: nil.
	self assert: browser classList isEmpty.! !

!PackagePaneBrowserTest methodsFor: 'tests' stamp: 'fbs 3/16/2011 07:37'!
testPackageListIndex
	browser packageListIndex: 0.
	self assert: browser selectedClass isNil.
	
	browser selectCategoryForClass: browser class.
	
	"Clicking the selected package unselects the selected class."
	browser packageListIndex: browser packageListIndex.
	self assert: browser selectedClass isNil.! !

!PackagePaneBrowserTest methodsFor: 'tests' stamp: 'fbs 6/11/2012 14:25'!
testSelectCategoryForClass
	| pkgName |
	pkgName := browser class category copyUpTo: $-.
	browser selectCategoryForClass: browser class.
	self assert: browser selectedSystemCategory = browser class category.
	self assert: browser packageListIndex = (browser packageList indexOf: pkgName).
	browser selectSystemCategory: nil.
	self assert: browser classList isEmpty.! !

Object subclass: #DummyToolWorkingWithFileList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-FileList'!

!DummyToolWorkingWithFileList commentStamp: '<historical>' prior: 0!
I'm a dummy class for testing that the registration of the tool to the FileList of actually happens.
In the future the tests should cover that the class register when loaded in memory and unregister when unloaded.!

!DummyToolWorkingWithFileList class methodsFor: 'class initialization' stamp: 'sd 2/6/2002 21:29'!
fileReaderServicesForFile: fullName suffix: suffix

	^ (suffix = 'kkk')
		ifTrue: [ self services]
		ifFalse: [#()] ! !

!DummyToolWorkingWithFileList class methodsFor: 'class initialization' stamp: 'tpr 1/7/2019 15:40'!
initialize
	"self initialize"

	FileServices registerFileReader: self

! !

!DummyToolWorkingWithFileList class methodsFor: 'class initialization' stamp: 'SD 11/14/2001 22:12'!
loadAFileForTheDummyTool: aFileListOrAPath
	
	"attention. if the file list selects a file the argument will be a fullpath of the selected file else it will pass the filelist itself"! !

!DummyToolWorkingWithFileList class methodsFor: 'class initialization' stamp: 'sw 2/17/2002 02:36'!
serviceLoadAFilForDummyTool
	"Answer a service for opening the Dummy tool"

	^ SimpleServiceEntry 
		provider: self 
		label: 'menu label'
		selector: #loadAFileForTheDummyTool:
		description: 'Menu label for dummy tool'
		buttonLabel: 'test'! !

!DummyToolWorkingWithFileList class methodsFor: 'class initialization' stamp: 'sd 2/1/2002 22:32'!
services 

	^ Array with: self serviceLoadAFilForDummyTool

! !

!DummyToolWorkingWithFileList class methodsFor: 'class initialization' stamp: 'tpr 1/7/2019 15:40'!
unload

	FileServices unregisterFileReader: self ! !

!DummyToolWorkingWithFileList class methodsFor: 'class initialization' stamp: 'tpr 1/7/2019 15:40'!
unregister

	FileServices unregisterFileReader: self.
	! !

Object variableSubclass: #InspectorTestObject
	instanceVariableNames: 'apple orange'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolsTests-Inspector'!

!InspectorTestObject class methodsFor: 'instance creation' stamp: 'mt 4/8/2020 14:25'!
newInvalid
	"Creates an instance that cannot #printOn: without raising an error."

	^ self new
		beInvalid;
		yourself! !

!InspectorTestObject methodsFor: 'accessing' stamp: 'mt 4/7/2020 17:46'!
apple

	^ apple! !

!InspectorTestObject methodsFor: 'accessing' stamp: 'mt 4/8/2020 14:06'!
apple: aSymbol

	apple := aSymbol.! !

!InspectorTestObject methodsFor: 'initialization' stamp: 'mt 4/8/2020 14:25'!
beInvalid

	self apple: 5.! !

!InspectorTestObject methodsFor: 'accessing' stamp: 'mt 4/7/2020 17:49'!
fruits

	^ {self apple. self orange} select: [:fruit | fruit notNil and: [fruit size > 0]]! !

!InspectorTestObject methodsFor: 'accessing' stamp: 'mt 4/7/2020 17:46'!
orange

	^ orange! !

!InspectorTestObject methodsFor: 'accessing' stamp: 'mt 4/8/2020 14:07'!
orange: aSymbol

	orange := aSymbol.! !

!InspectorTestObject methodsFor: 'printing' stamp: 'mt 4/7/2020 17:48'!
printOn: aStream

	aStream nextPutAll: 'Today''s offers: '.
	self fruits do: [:fruit | aStream print: fruit].! !
DummyToolWorkingWithFileList initialize!
